function [J,maxdiff,stim] = adblurnans2(im,mask,itwid,tol,maxiter,force2d)

% remove NaNs by iteratively blurring the image until it no longer changes
% put in variable input arguments later..
if nargin<6 || isempty(force2d)
    force2d = 0;
end
if nargin<5 || isempty(maxiter)
    maxiter = 50;
end

if nargin<4 || isempty(tol)
    tol = 1e-2;
end
if nargin<3 || isempty(itwid)
    itwid = 2;
end
if nargin<2 || isempty(mask)
    mask = double(isnan(im));
else
    mask(isnan(im)) = 1;
end

ndim = numel(size(im));

temp1 = im;
temp1(isnan(im)) = 0;
% with a grayscale mask, have be more careful here..

meanval = sum(temp1(:).*(1-mask(:)))/sum(1-mask(:));

temp = meanval*mask + temp1.*(1-mask);

bigfi = dog3split2(5*itwid,100);
smallfi = dog3split2(itwid,8*itwid);

stim = sqrt(sum((1-mask(:)).*(temp1(:)-meanval).^2)/sum(1-mask(:)));

if ndim==2 || force2d
    J = gauss2filt(temp,bigfi);
else
    J = gauss3filt(temp,bigfi);
end

% then reset the good pixel values
J = mask.*J + (1-mask).*temp1;

for ii = 1:maxiter
%     ii
    oldJ = J;
    
    if ndim==2 || force2d
        J = gauss2filt(J,smallfi);
    else
        J = gauss3filt(J,smallfi);
    end
    J = mask.*J + (1-mask).*temp1;
    
    maxdiff(ii) = max(abs(J(:)-oldJ(:)));
    if maxdiff(ii)<(tol*stim)
        break
    end
    
end



