function L3 = add3Dlabels(L2,im,gradim,vectgrad)

% Take the 2D label matrix and expand each column into 3D
% The z gradient particularly gets very small around the edges - unless
% something can be done about this we might have to artificially impose
% smaller heights 1 or two pixels from the boundary..

if nargin<4 || isempty(vectgrad)
    vectgrad = 0; % assume we only have the absolute value of the gradient by default, not its direction.
end

tstats = regionprops(L2,'Centroid','PixelIdxList');
% The centroid isn't guaranteed to fall inside the region!

bg = bwmorph(L2==0,'skel',Inf);

nucl = im(:,:,1);
bgl = admean(nucl(bg));

nucu = im(:,:,end);
bgu = admean(nucu(bg));

if nargin<3 || isempty(gradim)
    temp = cat(3,bgl*ones(size(L2)),im,bgu*ones(size(L2)));
    gradim = temp(:,:,3:end) - temp(:,:,1:end-2);
end

L3 = zeros(size(im));

xyz = zeros(numel(tstats),3);
for ii = 1:numel(tstats)
    if isnan(tstats(ii).Centroid(1))
        % must a region which has disappeared during various morphological
        % operations.  Would be good to have a record to see if any manage
        % to get through to here..
        continue
    end
    
% %     nuclabel = L2(round(tstats(ii).Centroid(2)),round(tstats(ii).Centroid(1)));
    % maybe a better way would be to find the nearest pixel to the
    % centroid, which would work either way.
    
    allxy = adind2sub(size(L2),tstats(ii).PixelIdxList(round(numel(tstats(ii).PixelIdxList)/2)));
    d2 = (allxy(:,1) - tstats(ii).Centroid(2)).^2 + (allxy(:,2) - tstats(ii).Centroid(1)).^2;
    tempxy = allxy(find(d2==min(d2),1,'first'),:);
    
% %     if nuclabel~=ii
% %         warning('Weird shaped region')
% % %         keyboard
% %         tempxy = adind2sub(size(L2),tstats(ii).PixelIdxList(round(numel(tstats(ii).PixelIdxList)/2)));
% %     else
% %         tempxy = round(tstats(ii).Centroid([2,1]));
% %     end
    try
        smoothint = adsmooth(squeeze(im(tempxy(1),tempxy(2),:)),5);
    catch ME
        rethrow(ME)
    end
    xyz(ii,:) = [tempxy, find(smoothint==max(smoothint),1,'first')];
  
    nuclabel = L2(xyz(ii,1),xyz(ii,2));
    thisN = L2==nuclabel;
    slim = reshape(im(repmat(thisN,[1 1 size(im,3)])),[nnz(thisN),size(im,3)]);
    
    slimgrad = reshape(gradim(repmat(thisN,[1 1 size(im,3)])),[nnz(thisN),size(im,3)]);
    
    
    % optional smoothing of the gradient in the horizontal direction.
    fi = dog3split(12,50);
    slimgrad = imfilter(slimgrad,fi.x1,'same','replicate');
    
    
    smslim = adordfilt(slim,true(3,5),0.5);
    % WHICH IS BETTER?
    
    slimbw = zeros(size(smslim));
    maxvals = zeros(size(smslim,1),1);
    % find the maximal gradients either side of the maxima
    zz = 1:size(smslim,2);
    for kk = 1:size(smslim,1)
        maxvals(kk) = find(smslim(kk,:)==max(smslim(kk,:)),1,'first');
    end
    
    % having a single centre slice might be good enough for most cells, but
    % metaphase DNA can be tilted, in which case we should pick the max of
    % each column as being the centre slice
    centreslice = admean(maxvals);
    
    % replace extreme maxima with the centre slice
    maxvals(maxvals==1 | maxvals==zz(end)) = centreslice;
    
    usecent = 1*centreslice + 0*maxvals;
    
    for kk = 1:size(smslim,1)
        
        sgrad = slimgrad(kk,:);
        if usecent(kk)>1
            lval = max(sgrad(zz<usecent(kk)));
            lind = find(zz<usecent(kk) & sgrad==lval,1,'first');
        else
            lind = 1;
        end
        
        if usecent(kk)<=(size(smslim,2)-1)
            if vectgrad
                uval = min(sgrad(zz>usecent(kk)));
            else
                uval = max(sgrad(zz>usecent(kk)));
            end
            uind = find(zz>usecent(kk) & sgrad==uval,1,'last');
        else
            uind = size(smslim,2);
        end
        
        slimbw(kk,lind:uind) = ii;
    end
    
%     slimbw = ii*(smslim>(nucv-0.8*nucstd));

    L3(repmat(thisN,[1,1,size(im,3)])) = slimbw(:);
%     maxL(repmat(thisN,[1,1,size(im,3)])) = slimbw2(:);
    
    
    % check for fragmentation
    [temp,tN] = bwlabeln(L3==ii);
    if tN>1
        % keep only the part with the largest volume
        fragstats = regionprops(temp,'Area');
        keepind = find([fragstats.Area]==max([fragstats.Area]),1,'first');
        L3(L3==ii)=0;
        L3(temp==keepind)=ii;
%         disp('Sorted out fragmented nucleus') % to see how prevalent this is
        % might be better to write to a log instead of the screen
    end
end
