function [gx,gy,gz] = adgaussgrad(im,sigma,siz)

if nargin<2 || isempty(sigma)
    sigma = [1,1,1];
end

if isscalar(sigma)
    sigma = [1,1,1]*sigma;
end

if nargin<3 || isempty(siz)
    siz = max(9,odd(5*sigma,'up'));
end

if isscalar(siz)
    siz = [1,1,1]*siz;
end

% the kernels are separable for each dimension, but we need 6 different
% filters since the widths can be different in each direction

im = double(im);

xx = ((1:siz(1)) - (siz(1)+1)/2)';
yy = ((1:siz(2)) - (siz(2)+1)/2);
zz = permute(((1:siz(3)) - (siz(3)+1)/2),[1,3,2]);

fx = exp(-xx.^2/(2*sigma(1)^2)) / (sigma(1)*sqrt(2*pi));
fy = exp(-yy.^2/(2*sigma(2)^2)) / (sigma(2)*sqrt(2*pi));
fz = exp(-zz.^2/(2*sigma(3)^2)) / (sigma(3)*sqrt(2*pi));

dfx = -xx/sigma(1)^2 .* fx;
dfy = -yy/sigma(2)^2 .* fy;
dfz = -zz/sigma(3)^2 .* fz;

% now ready to do the filtering
% check that imfilter can handle negatives, might just be the uint types
% that cause problems

bx = imfilter(im,fx,'same','replicate');

% bx forms the base of gy and gz
gy = imfilter(imfilter(bx,fz,'same','replicate'),dfy,'same','replicate');
gz = imfilter(imfilter(bx,fy,'same','replicate'),dfz,'same','replicate');

gx = imfilter(imfilter(imfilter(im,fy,'same','replicate'),fz,'same','replicate'),dfx,'same','replicate');

if nargout==1
    % most likely scenario is that we only want the magnitude of the
    % gradient
    gx = sqrt(gx.^2 + gy.^2 + gz.^2);
end

