function [gx,gy,gbase] = adgaussgrad2(im,sigma,siz)

% x and y gradients
% the magnitude of the gradient is given by sqrt(gx.^2 + gy.^2)
% the direction of the gradient is atan2(gy,gx)

% the width of the smoothing
if nargin<2 || isempty(sigma)
    sigma = [1,1];
end

if isscalar(sigma)
    sigma = [1,1]*sigma;
end

% the size of the filter array (use at least 5x width)
if nargin<3 || isempty(siz)
    siz = max(9,odd(5*sigma,'up'));
end

if isscalar(siz)
    siz = [1,1]*siz;
end

% the kernels are separable for each dimension, but we need 6 different
% filters since the widths can be different in each direction

im = double(im);

xx = ((1:siz(1)) - (siz(1)+1)/2)';
yy = ((1:siz(2)) - (siz(2)+1)/2);

fx = exp(-xx.^2/(2*sigma(1)^2)) / (sigma(1)*sqrt(2*pi));
fy = exp(-yy.^2/(2*sigma(2)^2)) / (sigma(2)*sqrt(2*pi));

dfx = -xx/sigma(1)^2 .* fx;
dfy = -yy/sigma(2)^2 .* fy;

% now ready to do the filtering
% check that imfilter can handle negatives, might just be the uint types
% that cause problems

% bx forms the base of gy and gz
gy = imfilter(imfilter(im,fx,'same','symmetric'),dfy,'same','symmetric');
gx = imfilter(imfilter(im,fy,'same','symmetric'),dfx,'same','symmetric');


if nargout==1
    % most likely scenario is that we only want the magnitude of the
    % gradient
    gx = sqrt(gx.^2 + gy.^2);
end

if nargout>2
    gbase = imfilter(imfilter(im,fx,'same','symmetric'),fy,'same','symmetric');
end
