function bw = adthresh(im,nhood,offset,method)

% N-dimensional adaptive thresholding

% two possible methods
% - subtract off the mean of the neighbourhood and threshold at the offset
% - use the neighbourhood to calculate the threshold level in the same way
%   as graythresh.

% imfilter is supposed to automatically detect and implement separable
% 1D filtering where possible, but still seems very slow.
% modify so that when nhood is specified as dimensions rather than the
% actual kernel, we perform 3 1D filters rather than 1 3D one.

if nargin<4 || isempty(method)
    method = 1;
end
if nargin<3 || isempty(offset)
    offset = 0;
end
if nargin<2 || isempty(nhood)
    nhood = ones(5,5);
end

switch method
    case 1
        % subtract off mean
        if (numel(nhood)==2 || numel(nhood)==3 || numel(nhood)==4) && ~islogical(nhood)
            % cuboidal neighbourhood, separable into 1D filters for speed.
            fim1 = imfilter(im,ones(nhood(1),1)/nhood(1),'symmetric','same');
            subim = imfilter(fim1,ones(1,nhood(2))/nhood(2),'symmetric','same');
            if numel(nhood)>2
                subim = imfilter(subim,ones(1,1,nhood(3))/nhood(3),'symmetric','same');
                if numel(nhood)>3
                    subim = imfilter(subim,ones(1,1,1,nhood(4))/nhood(4),'symmetric','same');
                end
            end
        else
            subim = imfilter(im,nhood/nnz(nhood),'symmetric','same');
        end
    case 2
        % subtract off Otsu level
        subim = nlfilter(im,size(nhood),@graythresh);
        % doesn't seem to work properly!
end

bw = (im - subim)>offset;