function varargout = adthreshstd(im,nhood,thresh)

% N-dimensional adaptive thresholding

% two possible methods
% - subtract off the mean of the neighbourhood and threshold at the offset
% - use the neighbourhood to calculate the threshold level in the same way
%   as graythresh.

% this std version also calculates the standard deviation of the region and
% the threshold is the number of standard deviations that the pixel must be
% above the mean to be on.

% imfilter is supposed to automatically detect and implement separable
% 1D filtering where possible, but still seems very slow.
% modify so that when nhood is specified as dimensions rather than the
% actual kernel, we perform 3 1D filters rather than 1 3D one.

if nargin<3 || isempty(thresh)
    thresh = 0;
end
if nargin<2 || isempty(nhood)
    nhood = [5 5];
end

% switch method
%     case 1
        % subtract off mean
        if (numel(nhood)==2 || numel(nhood)==3 || numel(nhood)==4) && ~islogical(nhood)
            % cuboidal neighbourhood, separable into 1D filters for speed.
            fim1 = imfilter(im,ones(nhood(1),1)/nhood(1),'symmetric','same');
            fim2 = imfilter(im.^2,ones(nhood(1),1)/nhood(1),'symmetric','same');
            
            subim = imfilter(fim1,ones(1,nhood(2))/nhood(2),'symmetric','same');
            subim2 = imfilter(fim2,ones(1,nhood(2))/nhood(2),'symmetric','same');
            if numel(nhood)>2
                subim = imfilter(subim,ones(1,1,nhood(3))/nhood(3),'symmetric','same');
                subim2 = imfilter(subim2,ones(1,1,nhood(3))/nhood(3),'symmetric','same');
                
                if numel(nhood)>3
                    subim = imfilter(subim,ones(1,1,1,nhood(4))/nhood(4),'symmetric','same');
                    subim2 = imfilter(subim2,ones(1,1,1,nhood(4))/nhood(4),'symmetric','same');
                end
            end
        else
            subim = imfilter(im,nhood/nnz(nhood),'symmetric','same');
            subim2 = imfilter(im.^2,nhood/nnz(nhood),'symmetric','same');
            
        end
%     case 2
%         % subtract off Otsu level
%         subim = nlfilter(im,size(nhood),@graythresh);
%         % doesn't seem to work properly!
% end

% varim = subim2 - subim.^2;

testim = (im-subim)./sqrt(subim2 - subim.^2);

if ~isnan(sum(thresh))
    varargout{1} = testim > thresh(1);
    
    for ii = 2:numel(thresh)
        varargout{ii} = testim > thresh(ii);
    end
else
    varargout{1} = testim;
end

