function D = constructcostmat(prevcoords,nextcoords,cutoff,fovlim,efunc,dfunc,nonull)

if nargin<7 || isempty(nonull)
    nonull = 0;
end
if nargin<6
    dfunc = [];
end
if nargin<5
    efunc = [];
end

if nargin<4 || isempty(fovlim)
    fovlim = [1,1;512,512];
end

% by default, add sufficient null objects to allow 1/4 of the objects to
% disappear or reappear.  Hopefully with the null-null cost set to zero,
% the actual number of nulls shouldn't be a factor..

pobj = size(prevcoords,1);
nobj = size(nextcoords,1);

if nonull
    pnull = 0;
    nnull = 0;
else
    pnull = max(2,ceil(nobj/4));
    nnull = max(2,ceil(pobj/4));
end

rawD = sum((repmat(permute(prevcoords,[1,3,2]),[1,nobj,1]) - repmat(permute(nextcoords,[3,1,2]),[pobj,1,1])).^2,3);
if ~isempty(dfunc)
    rawD = dfunc(sqrt(rawD));
end

if nonull
    D = rawD;
    return
end

pedist = min(cutoff,calcedgedist(prevcoords,fovlim));
nedist = min(cutoff,calcedgedist(nextcoords,fovlim));

if isempty(efunc)
    pcost = cutoff*pedist;
    ncost = cutoff*nedist;
else
    pcost = efunc(pedist);
    ncost = efunc(nedist);
end

D = [rawD,(pcost*ones(1,nnull));(ones(pnull,1)*ncost'),zeros(pnull,nnull)];