function d = distance(p1,p2,dim)

% Keeping with matlab convention, dim is the dimension which disappears, ie
% the dimension which is to be summed over.

if nargin<2 || isempty(p2)
	p2=zeros(size(p1));
end
if nargin<3 & numel(p1)==numel(p2) & size(p1)~=size(p2)
	p2 = reshape(p2,size(p1));
end

% If dim is not specified, we have to guess it from the size of the input
% vectors; if one of the dimensions has size 2 or 3, then we'll take that
% as dim

if nargin<3 || isempty(dim)
	siz = size(p1);
	inds = find(siz==2 | siz==3);
	if isempty(inds)
		dim = length(siz);
	else
		dim = inds(end);
		% Choose the last dimension with the desired dimension
	end
end

if numel(p1)~=numel(p2)
    % One the p1,p2 should be a single value which we want to calculate the
    % distances of all of the other.
    x1 = size(p1,1);
    y1 = size(p1,2);
    x2 = size(p2,1);
    y2 = size(p2,2);
    if x1==1
        p1 = repmat(p1,[x2 1]);
        dim = 2;
    elseif y1==1
        p1 = repmat(p1,[1 y2]);
        dim = 1;
    elseif x2==1
        p2 = repmat(p2,[x1,1]);
        dim = 2;
    elseif y2==1
        p2 = repmat(p2,[1 y1]);
        dim = 1;
    else
        error('Sizes of input arrays don''t match up')
    end
end


d = sqrt(sum((p1-p2).^2 , dim));