function imout = dog2filt(im,fi,doratio)

% Custom written filtering for the DoG filter, fi, which has been split
% into 1D positive and negative components.

% fi.x1 is the positive (narrower) Gaussian
% fi.x2 is the negative (wider) Gaussian

% Check the speeds later, but for now permute the filter rather than the
% image for y and z.

% On first inspection, this seems to work exactly the same as the 3d version using
% convn.m, but is much faster and also doesn't suffer as much from edge
% effects.

if nargin<3 || isempty(doratio)
    doratio = 0;
end

if ~isa(im,'double')
	im = double(im);
end

if ~isstruct(fi)
    fi = dog3split(fi,6*fi);
end

% imfilter is fast but cannot handle negative values.  Some scripts use a
% negative version of the DoG filter to find dark objects.  That needs
% sorting here!
% THE ABOVE IS ONLY TRUE IF WE'RE USING UINT ARRAYS!

pos = max(sign(max(fi.x1)),0);
% return 1 if positive, 0 if negative

if pos
    imf1 = imfilter(im,fi.x1,'replicate');
    imf1 = imfilter(imf1,permute(fi.y1,[2 1]),'replicate');
    % imf1 = imfilter(imf1,permute(fi.z1,[2 3 1]),'replicate');

    imf2 = imfilter(im,fi.x2,'replicate');
    imf2 = imfilter(imf2,permute(fi.y2,[2 1]),'replicate');
    % imf2 = imfilter(imf2,permute(fi.z2,[2 3 1]),'replicate');
else
    imf1 = imfilter(im,fi.x1,'replicate');
    imf1 = -imfilter(imf1,permute(fi.y1,[2 1]),'replicate');
    % imf1 = imfilter(imf1,permute(fi.z1,[2 3 1]),'replicate');

    imf2 = imfilter(im,fi.x2,'replicate');
    imf2 = -imfilter(imf2,permute(fi.y2,[2 1]),'replicate');
    % imf2 = imfilter(imf2,permute(fi.z2,[2 3 1]),'replicate');
end

imout = imf1 - imf2;

if doratio
    imout = imout./imf2;
end

