function fi = dog3split(width,siz,factor,oldver)

% splitting the DoG filter into
% positive and negative parts, then decomposing into 1D filters.

if length(width)==1
	width = [1 1 1]*width;
end

if nargin<4 || isempty(oldver)
    oldver = 0; % allow the old version to be run for compatibility..
end

if nargin<3 || isempty(factor)
	factor = 1.6;
end
if nargin<2 || isempty(siz)
	siz = ceil(4*width);
end

siz = odd(siz,'up');

if isscalar(siz)
    siz = [1 1 1]*siz;
end


n = (1:siz(1))';
m = (1+siz(1))/2;
width2 = factor*width(1);

% a more accurate representation of narrow kernels would be to sum up a
% bunch of values to approximate an integral

fi.x1 = (2*pi*width(1)^2)^(-1/2) * exp(-((n-m).^2)/(2*width(1)^2));
fi.x2 = (2*pi*width2^2)^(-1/2) * exp(-((n-m).^2)/(2*width2^2));


n = (1:siz(2))';
m = (1+siz(2))/2;
width2 = factor*width(2);

fi.y1 = (2*pi*width(2)^2)^(-1/2) * exp(-((n-m).^2)/(2*width(2)^2));
fi.y2 = (2*pi*width2^2)^(-1/2) * exp(-((n-m).^2)/(2*width2^2));


n = (1:siz(3))';
m = (1+siz(3))/2;
width2 = factor*width(3);

fi.z1 = (2*pi*width(3)^2)^(-1/2) * exp(-((n-m).^2)/(2*width(3)^2));
fi.z2 = (2*pi*width2^2)^(-1/2) * exp(-((n-m).^2)/(2*width2^2));


if ~oldver
    % normalize so the kernels sum to 1, which isn't guaranteed for very
    % large and very small widths..
    
    fi.x1 = fi.x1/sum(fi.x1);
    fi.y1 = fi.y1/sum(fi.y1);
    fi.z1 = fi.z1/sum(fi.z1);
    fi.x2 = fi.x2/sum(fi.x2);
    fi.y2 = fi.y2/sum(fi.y2);
    fi.z2 = fi.z2/sum(fi.z2);
    
    
end
