function [v,d,cc] = eigvect(im,cc)

% Finds the eigendirections for a (presumably elliptical) image, either 2D
% or 3D.
%
% The 3D one will probably be quite slow, but this time it might actually
% work quite well.

sumim = sum(im(:));

if nargin<2 || isempty(cc)
%     cc = findn(im==max(im(:)));
    switch length(size(im))
        case 2
            [x,y] = makegrid(im);
        case 3
            [x,y,z] = makegrid(im);
    end
    cc(1) = sum(x(:).*im(:))/sumim;
    cc(2) = sum(y(:).*im(:))/sumim;
    if length(size(im))==3
        cc(3) = sum(z(:).*im(:))/sumim;
    end
end

switch length(size(im))
    case 2
        [x,y] = makegrid(im);
        x2 = sum((x(:)-cc(1)).^2.*im(:))/sumim;
        y2 = sum((y(:)-cc(2)).^2.*im(:))/sumim;
        xy = sum((x(:)-cc(1)).*(y(:)-cc(2)).*im(:))/sumim;
        
        rmat = [x2,xy;xy,y2];
        
    case 3
        [x,y,z] = makegrid(im);
        x2 = sum((x(:)-cc(1)).^2.*im(:))/sumim;
        y2 = sum((y(:)-cc(2)).^2.*im(:))/sumim;
        z2 = sum((z(:)-cc(3)).^2.*im(:))/sumim;
        
        xy = sum((x(:)-cc(1)).*(y(:)-cc(2)).*im(:))/sumim;
        xz = sum((x(:)-cc(1)).*(z(:)-cc(3)).*im(:))/sumim;
        yz = sum((y(:)-cc(2)).*(z(:)-cc(3)).*im(:))/sumim;
                
        rmat = [x2,xy,xz;xy,y2,yz;xz,yz,z2];
        
end

if nnz(isnan(rmat))==0
    [v,d] = eig(rmat);
else
    v = NaN*rmat;
    d = NaN*rmat;
end
