function val = field2val2(s,fname1,fname2,fname3)

% like field2val, but allows depths of up to 3

% find out if the field is a character array
if isempty(s)
    val = [];
    return
end

if nargin==2 && iscell(fname1)
    tnames = fname1;
    fname1 = tnames{1};
    if numel(tnames)>1
        fname2 = tnames{2};
        ndepth = 3;
        if numel(tnames)>2
            fname3 = tnames{3};
            ndepth = 4;
        end
    end
else
    if isnan(fname1)
        fname1 = fname2;
        ndepth = 2;
        if nargin>3 && ~isempty(fname3)
            fname2 = fname3;
            ndepth = 3;
        end
    else
        ndepth = nargin;
    end
end

fchar = NaN;

for jj = 1:numel(s)
    switch ndepth
        case 2
            temp2 = s(jj).(fname1);
        case 3
            temp2 = s(jj).(fname1).(fname2);
        case 4
            temp2 = s(jj).(fname1).(fname2).(fname3);
    end
    if ~isempty(temp2)
        fchar = ischar(temp2);
        break
    end
end

if isnan(fchar)
    % all of the fields are empty, so we should return an array of NaNs
    val = NaN*ones(size(s));
    return
end

if fchar
    % the field is a string, so we need to initialise it differently
    val = cell(numel(s),1);

else
    for jj = 1:numel(s)
        % first find the maximum dimensions for the required field
        switch ndepth
            case 2
                if jj==1
                    siz = size(s(1).(fname1));
                else
                    siz = max(siz,size(s(jj).(fname1)));
                end
            case 3
                if jj==1
                    siz = [size(s(1).(fname1)) size(s(1).(fname1).(fname2))];
                else
                    try
                        % need to check first that fname1 field isn't empty
                        if ~isempty(s(jj).(fname1))
                            siz = max(siz,[size(s(jj).(fname1)) size(s(jj).(fname1).(fname2))]);
                        end
                        
                    catch ME
                        rethrow(ME)
                    end
                end
            case 4
                if jj==1
                    siz = [size(s(1).(fname1)) size(s(1).(fname1).(fname2)) size(s(1).(fname1).(fname2).(fname3))];
                else
                    siz = max(siz,[size(s(jj).(fname1)) size(s(jj).(fname1).(fname2)) size(s(jj).(fname1).(fname2).(fname3))]);
                end
        end
    end

    slen = numel(s);
    siz(siz==1) = [];
    if isempty(siz)
        siz = 1;
    end
    valsiz = [slen siz];


    % make a temp then reshape it to incorporate into val
    val = NaN*zeros(valsiz);
end

for ii = 1:numel(s)
    switch ndepth
        case 2
            temp = squeeze(s(ii).(fname1));
        case 3
            try
                temp = squeeze(s(ii).(fname1).(fname2));
            catch ME
                % this is faster than prechecking every sub-structure's
                % fieldnames, but can it go wrong?
%                 rethrow(ME)
                continue
            end
        case 4
            temp = squeeze(s(ii).(fname1).(fname2).(fname3));
    end
%     tempdim = length(size(temp));
%     temp = permute(temp,[tempdim+1 1:tempdim]);
    tempsiz = size(temp);
    if ~ischar(temp)
        if tempsiz(1)==1
            temp = permute(temp,[2:length(tempsiz) 1]);
        end
        switch numel(size(temp))
            case 2
                for jj = 1:size(temp,1)
                    for kk = 1:size(temp,2)
                        val(ii,jj,kk) = temp(jj,kk);
                    end
                end
            case 3
                for jj = 1:size(temp,1)
                    for kk = 1:size(temp,2)
                        for ll = 1:size(temp,3)
                            val(ii,jj,kk,ll) = temp(jj,kk,ll);
                        end
                    end
                end
            case 4
                for jj = 1:size(temp,1)
                    for kk = 1:size(temp,2)
                        for ll = 1:size(temp,3)
                            for mm = 1:size(temp,4)
                                val(ii,jj,kk,ll) = temp(jj,kk,ll);
                            end
                        end
                    end
                end
        end
    else
        val{ii} = temp;
    end
end
if ~fchar
    val = reshape(val,[numel(s) siz]);
end
