function goodtracks = findgoodtracks(cellprops,thresh)

% find and record the indices of cells which are unambiguously tracked.
% For now, we'll be very strict and say that only one-to-one prevs and nexts are
% allowed.  If we want to relax this, we could look at nextnexts and
% prevprevs to reduce the effect of noise.

% goodtracks = cell(0); % worth doing this?
% the alternative is to have an array of good tracks where the vertical
% dimension is frame and the horizontal is track number, NaNs where no
% objects exists.

if nargin<2 || isempty(thresh)
    thresh = 3; % threshold number of objects in a track for it to be recorded.
end


tcount = 0;
freeinds = true(numel(cellprops),1);

while nnz(freeinds)>0
    startind = find(freeinds,1,'first');
    % start with the first unused index
    
    freeinds(startind) = false;
    temptrack = startind;
    
    nextind = cellprops(startind).nextcells;
%     try
    while numel(nextind)==1 && numel(cellprops(nextind).prevcells)==1
        temptrack = [temptrack; nextind];
        freeinds(nextind) = false;
        nextind = cellprops(nextind).nextcells;
    end
%     catch ME
%         rethrow(ME)
%     end
    % temptrack now lists the cell indices of a track, if it's long enough
    % add it to goodtracks
    if numel(temptrack)>=thresh
        tcount = tcount + 1;
        goodtracks(1:numel(temptrack),tcount) = temptrack;
    end
end

if tcount==0
    goodtracks = [];
end




