function spotdata = gauss3spotdata(im,xyz,spotsig)

% return parameters obtained by fitting Gaussian profiles to the
% potentially identified spots with positions xyz in the image im.
%
% Use the spotsiz variable to initialize the fitting and to determine what
% size of window we're going to cut out.

% We also need to decide whether to fit in 3D or 2D.  I suppose 3D is
% technically better, but might also be more likely to fail during the
% fitting step?  Perhaps a failure to fit properly could be a sign of a bad
% spot anyway?

% Use 2D for now and implement a separate 3D version
% Also want to construct a function which will perform the existing spot
% intensity measurements (ie using concentric cylinders) for comparison.

if nargin<3 || isempty(spotsig)
    spotsig = 1.6;
end

% by default make the zsigma twice the x-y for initialization.

regsiz = odd([4,4,3]*spotsig,'up');
wwid = 2*spotsig; % can make bigger if necessary..

% First step, obtain the image regions
imreg = cell2mat(reshape(getimreg(im,xyz,regsiz),[1,1,1,size(xyz,1)]));

% for speed can define the x and w arrays outside the loop so they only
% need creating once.
xdata = adind2sub(regsiz,1:prod(regsiz));

wdata = exp(-((xdata(:,1)-(regsiz(1)+1)/2).^2 + (xdata(:,2)-(regsiz(1)+1)/2).^2)/(2*wwid^2));

afit = zeros(size(xyz,1),7);
r2 = zeros(size(xyz,1),1);

for ii = 1:size(xyz,1)
    [afit(ii,:),r2(ii,1)] = gausswfit3d(imreg(:,:,:,ii),xdata,[],wdata);
end

% now want to record the data in a format that's decipherable
for ii = 1:size(xyz,1)
    spotdata(ii,1).g3roughxyz = xyz(ii,:);
    spotdata(ii,1).g3fitxyz = xyz(ii,:) - (regsiz+1)/2 + afit(ii,2:4);
    
    spotdata(ii,1).g3fitr2 = r2(ii);
    
    spotdata(ii,1).g3fitsigxy = afit(ii,5);
    spotdata(ii,1).g3fitsigz = afit(ii,6);
    
    spotdata(ii,1).g3fitamp = afit(ii,1);
    spotdata(ii,1).g3fitbg = afit(ii,7);
    
    spotdata(ii,1).g3fitint = (2*pi)^(3/2)*afit(ii,1)*afit(ii,4).^2*afit(ii,5);
    % The 2D intensity should match pretty well, since the z-component will
    % be present in the amplitude (should find that A scales with zsig.
end
