function [afit,r2] = gausswfit2d(ydata,xdata,a0,wwid,options)

% fit a 2D Gaussian to the data in the ydata array
% to begin with, reshape everything within this function

% This weighted version allows the central pixels to be weighted more
% strongly than peripheral ones.  To do this we need to switch from
% lsqcurvefit to lsqnonlin..
% This means calculating the difference between the fit and the data,
% rather than just the fit.

if isvector(ydata)
    siz = sqrt(numel(ydata));
else
    siz = size(ydata,1);
    ydata = ydata(:);
end

if nargin<2 || isempty(xdata)
    xdata = adind2sub([siz,siz],1:(siz^2));
end

if nargin<4 || isempty(wwid)
    if nargin<3 || isempty(a0)
        wwid = siz/2;
    else
        % use our supplied sigma as a guide
        wwid = 2*a0(4);
    end
end

if numel(wwid)==1
    % generate a weighting mask then flatten it
    wdata = exp(-((xdata(:,1)-(siz+1)/2).^2 + (xdata(:,2)-(siz+1)/2).^2)/(2*wwid^2));
else
    wdata = wwid(:);
end

if nargin<3 || isempty(a0)
    % Parameters:
    % 1 - amplitude
    % 2 - x-centre
    % 3 - y-centre
    % 4 - sigma
    % 5 - background level (flat)
    bg = admean(ydata(min(xdata,[],2)==1 | max(xdata,[],2)==siz));
    a0 = [max(ydata) - bg, (siz+1)/2, (siz+1)/2, siz/4, bg];
end

if nargin<5 || isempty(options)
    options = optimset('Algorithm','levenberg-marquardt','display','off');
end

if nnz(isnan(ydata))>0
    xdata = xdata(~isnan(ydata),:);
    wdata = wdata(~isnan(ydata));
    ydata = ydata(~isnan(ydata));
end

fun = @(a) (a(1)*exp(-((xdata(:,1)-a(2)).^2 + (xdata(:,2)-a(3)).^2)/(2*a(4)^2))+a(5) - ydata).*wdata;

try
    [afit,resnorm] = lsqnonlin(fun,a0,[],[],options);
catch ME
    keyboard
end

ytot = sum((ydata-mean(ydata)).^2);
r2 = 1 - resnorm/ytot;
