function [afit,r2] = gausswfit3d(ydata,xdata,a0,wwid,options)

% fit a 3D Gaussian to the data in the ydata array
% to begin with, reshape everything within this function

% This weighted version allows the central pixels to be weighted more
% strongly than peripheral ones.  To do this we need to switch from
% lsqcurvefit to lsqnonlin..
% This means calculating the difference between the fit and the data,
% rather than just the fit.

siz = size(ydata);
ydata = ydata(:);


if nargin<2 || isempty(xdata)
    xdata = adind2sub(siz,1:prod(siz));
end

if nargin<4 || isempty(wwid)
    if nargin<3 || isempty(a0)
        wwid = siz(1)/2;
    else
        % use our supplied sigma as a guide
        wwid = 2*a0(5);
    end
end

cc = (siz+1)/2;

if numel(wwid)==1
    % generate a weighting mask then flatten it
    wdata = exp(-((xdata(:,1)-cc(1)).^2 + (xdata(:,2)-cc(2)).^2 + (xdata(:,3)-cc(3)).^2)/(2*wwid^2));
else
    wdata = wwid(:);
end

if nargin<3 || isempty(a0)
    % Parameters:
    % 1 - amplitude
    % 2 - x-centre
    % 3 - y-centre
    % 4 - z-centre
    % 5 - xy-sigma
    % 6 - z-sigma
    % 7 - background level (flat)
    bg = admean(ydata(min(xdata,[],2)==1 | xdata(:,1)==siz(1) | xdata(:,2)==siz(2) | xdata(:,3)==siz(3)));
    a0 = [max(ydata) - bg, cc, siz(1)/4, siz(1)/3, bg];
end

if nargin<5 || isempty(options)
    options = optimset('Algorithm','levenberg-marquardt','display','off');
end

if nnz(isnan(ydata))>0
    xdata = xdata(~isnan(ydata),:);
    wdata = wdata(~isnan(ydata));
    ydata = ydata(~isnan(ydata));
end

fun = @(a) (a(1)*exp(-((xdata(:,1)-a(2)).^2/(2*a(5)^2) + (xdata(:,2)-a(3)).^2/(2*a(5)^2) + (xdata(:,3)-a(4)).^2/(2*a(6)^2)))+a(7) - ydata).*wdata;

try
    [afit,resnorm] = lsqnonlin(fun,a0,[],[],options);
catch ME
    keyboard
end

ytot = sum((ydata-mean(ydata)).^2);
r2 = 1 - resnorm/ytot;
