function povrayexport(fv,objname,filename)

% export the matlab mesh fv as a povray mesh2 object

if nargin<2 || isempty(objname)
    objname = 'meshobject';
end
if nargin<3 || isempty(filename)
    filename = 'matlabmesh.inc';
end

% we have to get rid of any NaNs in our surface.  Where do these come from?
% should they be zeros, or deleted?

% set to zero for now while I write a function to remove them and renumber
% each face index
% fv.vertices(isnan(fv.vertices)) = 0;

% povray can only handle triangles
fv = triangulate(removemeshnans(fv));

fid = fopen(filename,'wt') %#ok

% syntax

% % mesh2{
% %     vertex_vectors{
% %         N,
% %         <x,y,z>,
% %         ...
% %     }
% %     face_indices{
% %         M,
% %         <#1,#2,#3>,
% %         ...
% %     }
% % }

v = sprintf('vertex_vectors {\n%i,\n', size(fv.vertices,1));
v = [v sprintf('<%f, %f, %f>,\n', fv.vertices(:,[1,3,2])')];
v = v(1:(end-2)); % end-2 because \n becomes one character using sprintf
v = [v sprintf('\n}\n')];

% don't bother with normals for now

f = sprintf('face_indices {\n%i,\n', size(fv.faces,1));
f = [f sprintf('<%i, %i, %i>,\n', fv.faces'-1)]; % -1 because povray indices go from zero
f = f(1:end-2);
f = [f sprintf('\n}\n')];

fprintf(fid,['#declare ' objname ' = mesh2 {\n' v f '}\n']);

fclose(fid) %#ok
