function [logpim,logppix] = pprodfilt(zim,nhood,flag)

% first convert the z-values into probabilities.
% depending on the value of flag, we include positive deviations, negative
% deviations, or both.

if nargin<3 || isempty(flag)
    flag = 0; % include positive and negative deviations
end
if nargin<2 || isempty(nhood)
    nhood = 1; % just probability conversion, no combination of pixels
end

% decide which tails to look at
if flag>0
    zim(zim<0) = 0;
elseif flag<0
    zim(zim>0) = 0;
end

% convert to probabilities
logppix = log(1 - erf(double(abs(zim))/sqrt(2)));

infinds = ~isfinite(logppix);
if nnz(infinds>0)
    logppix(infinds) = 0.5*log(2/pi)-0.5*zim(infinds).^2 - log(abs(zim(infinds)));
end


% multiply probabilities in the neighbourhood by taking log, adding up (via
% convolution) then exponentiating

if numel(nhood)~=1
% use imfilter for now, maybe switch to using conv2 for consistency
    logpim = imfilter(-logppix,double(nhood));
else
    logpim = -logppix;
end