function varargout = props2tracks(cellprops,tracks,fname)

% output the cellprops field fname in the tracks format.

if isempty(tracks)
    tracks = (1:numel(cellprops))';
end


% if fname has a dot in it (or more than one), split the names into
% multiple fieldnames

% the cell bit below is a temporary fix while field2val2 and props2tracks
% are being rewritten.
if iscell(fname)
    tempfname = fname{1};
    for ii = 2:numel(fname)
        tempfname = [tempfname '.' fname{ii}];
    end
    fname = tempfname;
    dotinds = strfind(fname,'.');
else
    dotinds = strfind(fname,'.');
end

if isempty(dotinds)
    % find the maximum number of elements for the property first.
    dim2 = 0;
    for ii = 1:numel(tracks)
        if tracks(ii)>0
            dim2 = max(dim2,numel(cellprops(tracks(ii)).(fname)));
        end
    end

    tempout = NaN*zeros(numel(tracks),dim2);

    for ii = 1:numel(tracks)
        % create in vector form and then reshape each column at the end.
        if tracks(ii)>0
            temp = cellprops(tracks(ii)).(fname);
            tempout(ii,1:numel(temp)) = temp(:)';
        end
    end
else
    % split into multiple nested fieldnames
    % currently each field can only have size 1x1
    if numel(dotinds)==1
        fname1 = fname(1:(dotinds-1));
        fname2 = fname((dotinds+1):end);
        % find the maximum number of elements for the property first.
        dim2 = 0;
        for ii = 1:numel(tracks)
            if tracks(ii)>0
                if ~isempty(cellprops(tracks(ii)).(fname1))
                    dim2 = max(dim2,numel(cellprops(tracks(ii)).(fname1).(fname2)));
                end
            end
        end

        tempout = NaN*zeros(numel(tracks),dim2);

        for ii = 1:numel(tracks)
            % create in vector form and then reshape each column at the end.
            if tracks(ii)>0
                if ~isempty(cellprops(tracks(ii)).(fname1))
                    temp = cellprops(tracks(ii)).(fname1).(fname2);
                    tempout(ii,1:numel(temp)) = temp(:)';
                end
            end
        end
    else
        error('Not implemented yet')
    end
end

for ii = 1:dim2
    varargout{ii} = reshape(tempout(:,ii),size(tracks));
end

if nargout>dim2
    % more outputs requested than elements in the specified field
    % return empty arrays
    for ii = (dim2+1):nargout
        varargout{ii} = [];
    end
end