function delta = regsubpix(imreg,method)

% find the peak of a 3D image to subpixel accuracy
% for noisy images, smoothing or LoG filtering might be an idea.

% need to allow for NaNs here, which woudl occur if we are near to the edge
% of the image.

if nargin<2 || isempty(method)
    method = 1;
end

if size(imreg,3)==1
    imreg = repmat(imreg,[1,1,3]);
    removez = 1;
else
    removez = 0;
end

switch method
    case 1
        % 1D central differences
        
        % z-position
        if ~isnan(sum(imreg(2,2,:)))
            fz = (imreg(2,2,3) - imreg(2,2,1))/2;
            fzz = (imreg(2,2,1) - 2*imreg(2,2,2) + imreg(2,2,3));
            delta(3) = - fz/fzz;
        else
            % at least one of z directions has a NaN in it, all we can
            % really do is pick the maximum value in this direction.
            maxval = find(imreg(2,2,:)==max(imreg(2,2,:)));
            if numel(maxval)>1
                maxval = mean(maxval);
            end
            delta(3) = maxval - 2;
        end
        
        % y-position
        if ~isnan(sum(imreg(2,:,2)))
            fy = (imreg(2,3,2) - imreg(2,1,2))/2;
            fyy = (imreg(2,1,2) - 2*imreg(2,2,2) + imreg(2,3,2));
            delta(2) = - fy/fyy;
        else
            % at least one of z directions has a NaN in it, all we can
            % really do is pick the maximum value in this direction.
            maxval = find(imreg(2,:,2)==max(imreg(2,:,2)));
            if numel(maxval)>1
                maxval = mean(maxval);
            end
            delta(2) = maxval - 2;
        end
        
        % x-position
        if ~isnan(sum(imreg(:,2,2)))
            fx = (imreg(3,2,2) - imreg(1,2,2))/2;
            fxx = (imreg(1,2,2) - 2*imreg(2,2,2) + imreg(3,2,2));
            delta(1) = - fx/fxx;
        else
            % at least one of z directions has a NaN in it, all we can
            % really do is pick the maximum value in this direction.
            maxval = find(imreg(:,2,2)==max(imreg(:,2,2)));
            if numel(maxval)>1
                maxval = mean(maxval);
            end
            delta(1) = maxval - 2;
        end
end

if removez
    delta(3) = [];
end

