function C = repmath(A,B,op)

% tile A and/or B so that they are the same size before carrying out the
% operation specified.  This now includes logical opertations as well as
% arithmetic, and comparisons like max and min

% usually the size of one dimension of A and/or B will be 1, if it isn't
% this function might give some unexpected results.

if numel(size(A))~=numel(size(B)) || any(size(A)~=size(B))
    sizA = [1,1,1,1,1,1];
    sizB = [1,1,1,1,1,1];
    sizA(1:numel(size(A))) = size(A);
    sizB(1:numel(size(B))) = size(B);

    outsiz = max(sizA,sizB);
    outdim = max(2,find(outsiz~=1,1,'last'));
    sizA = sizA(1:outdim);
    sizB = sizB(1:outdim);
    outsiz = outsiz(1:outdim);

    Ascale = outsiz./sizA;
    Bscale = outsiz./sizB;

    A2 = repmat(A,Ascale);
    B2 = repmat(B,Bscale);
else
    A2 = A;
    B2 = B;
end

switch lower(op)
    case {'times','x','mult','multiply','product','*'}
        C = A2.*B2;
    case {'div','divide','/','fraction','quotient'}
        C = A2./B2;
    case {'add','plus','sum','+'}
        C = A2+B2;
    case {'subtract','minus','diff','difference','-'}
        C = A2-B2;
    case {'^','power','exp','exponent'}
        C = A2.^B2;
    case {'=','==','eq','equals','equal'}
        C = A2==B2;
    case {'<','lt','less','lessthan'}
        C = A2<B2;
    case {'<=','leq','=<'}
        C = A2<=B2;
        % things like min, max etc will also be useful
    case {'&','&&','and'}
        C = A2 & B2;
    case {'|','||','or'}
        C = A2 | B2;
    otherwise
        error('Operation not implemented yet')
end

