function [cellprops,Lcell] = segcell3(cellim,nucbw,nhoodrad,method3,method2)

% segmentation algorithm for 3D cell images.  At the moment, this won't
% work too well because the images aren't deconvolved, look for a way
% around this.

% for now, nucbw will be 2D and should be used for every cell z-slice.

if nargin<5 || isempty(method2)
    method2 = 'watershed';
end
if nargin<4 || isempty(method3)
    method3 = 'footprint';
end

if nargin<3 || isempty(nhoodrad)
    nhoodrad = 8;
end
if nargin<2
    nucbw = [];
end

nhood = discel(nhoodrad,1);
L1 = zeros(size(cellim));

% if strcmpi(method2,'new')
%     % put the code in a separate function?
%     L1 = segcellnew(cellim,nucbw,nhood);
% else
    % for now, simply use the 2D segmentation algorithm for each slice
    for ii = 1:size(cellim,3)
        switch method2
            case 'watershed'
                L1(:,:,ii) = segcell(cellim(:,:,ii),nucbw,nhood,0);
            case {'threshold','fast','quick'}
                L1(:,:,ii) = segcellfast(cellim(:,:,ii),nucbw,nhood);
        end

        % remove the one with the largest footprint (should be the background)

        % QUICK WAY - also gets rid of cells which touch the edge, which
        % arguably we should do anyway (can't know the volume), assuming the
        % segmentation of out-of-focus cells can be improved.
        L1(:,:,ii) = imclearborder(L1(:,:,ii)>0);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % If we want to look primarily at gene expression, we might want to
        % keep cells which touch the edge of the field of view, since they
        % can still have expression spots.  But we need to have some way of
        % labelling them, because we need to know that the volume and area
        % properties won't be accurate.
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    end
% end

Lsum = sum(L1,3);

switch method3
    case 'watershed'
        Lsoc = imopen(imclose(Lsum,discel(nhoodrad,1)),discel(nhoodrad,1));
        if ~isempty(nucbw)
            Lreg = watershed(imimposemin(-Lsoc,nucbw));
        else
            Lreg = watershed(-Lsoc);
        end
    case 'footprint'
        Lreg = bwlabeln(L1(:,:,1));
        % this is noisier than the watershed method, as anything that gets
        % picked up in this slice alone will still be included.
end

[x,y] = makegrid2(Lsum);

% also want sum of fluorescence intensity (or average?)
% L1 has 1s wherever the cell is; Lreg has the 2D footprint for each
% cell.
Lreg3 = repmat(Lreg,[1 1 size(L1,3)]).*L1;
    
intbg = median(cellim(Lreg3==0));
% check the speed of this, might be better off doing afterwards, especially
% if the background is spatially varying

for ii = 1:max(Lreg(:))
    switch method3
        case 'watershed'
            cellprops(ii).vol = sum(Lsum(Lreg==ii));
        case 'footprint'
            cellprops(ii).vol = nnz((L1.*Lreg3)==ii);
    end
    try
        cellprops(ii).cellxy(1,1) = sum(x(Lreg==ii).*Lsum(Lreg==ii))/cellprops(ii).vol;
    catch ME
        rethrow(ME)
    end
    cellprops(ii).cellxy(1,2) = sum(y(Lreg==ii).*Lsum(Lreg==ii))/cellprops(ii).vol;
    
    % also recording which slices the potential cell appears in for
    % validation purposes would be useful.
    
    % depending on the speed of L1.*Lreg3, could be precalculated (used
    % above)
    
    cellprops(ii).slices = squeeze(sum(sum((L1.*Lreg3)==ii,1),2));
    
    
    % for the intensity, we need to subtract the estimated background off
    % first.
    
    
    cellprops(ii).int = sum(cellim(Lreg3==ii)) - cellprops(ii).vol*intbg;
    
    % put any other properties that we want here
    % - some shape measurement?
    % - 2D footprint area
end

Lcell = Lreg3;

% [] % used as debugging point
