function [cellprops,Lcell] = segcell3full(cellim,nucbw,nhood,stdfact)

% nhood is the size of a cuboidal box that is roughly the size of a cell.

% segment the GFP cell channel in 3D.
% this will take some time to get it working properly and sufficiently
% fast.

if nargin<4 || isempty(stdfact)
    stdfact = 0.4;
end

if nargin<2 || isempty(nucbw)
    % as in previous version, need to come up with some way of marking
    % individual cells as foreground.
    
    %%%%% NOT DONE YET! %%%%%%%%%
    
else
    if size(nucbw,2)==2 || size(nucbw,2)==3
        fg = false(size(cellim));
        nucbw = round(nucbw);
        nucbw = max(1,nucbw);
        nucbw(:,1) = min(nucbw(:,1),size(cellim,1));
        nucbw(:,2) = min(nucbw(:,2),size(cellim,1));
        if size(nucbw,2)==3
            nucbw(:,3) = min(nucbw(:,3),size(cellim,1));
        end
        
        for ii = 1:size(nucbw,1)
            fg(nucbw(ii,1),nucbw(ii,2),nucbw(ii,3)) = true;
        end
    else
        fg = nucbw;
    end
end

% let's see if the adaptive thresholding is OK, or if we'll need to
% watershed the gradient.

% subtract off mean of each x-y plane
% this step might help because the mean intensity appears to increase as we
% move upwards, making the out-of-focus segmenting difficult.
mz = mean(mean(cellim,1),2);
im = cellim - repmat(mz,[size(cellim,1),size(cellim,2),1]);

% could median filter in the z-direction here if we want.

cellbwp5 = adthreshstd(im,nhood,stdfact);

mp5 = adimfilter(cellbwp5,[3,3,3]);

% close and open this (is opening necessary?)
% ************* Would it be an idea to pad the top and bottom with zeros
% before doing this??******************

cellbw = imopen(imclose(imclose(mp5,ones(5,5,3))>(20/27),ones(3,3,3)),ones(3,3,3));
%  cellbw = imopen(imclose(mp5>(20/27),ones(3,3,3)),ones(3,3,3));

%  thickim = sum(cellbw,3);
thickimsegreg = sum(mp5>(20/27),3);

% now need to use the foreground marker to divide any joined cells
% watershedding of the gradient would segment the cells, but
% straightforward watershedding will still split joined cells.

%  Lreg = watershed(imimposemin(-thickim,max(fg,[],3)));
try
    Lreguse = watershed(imimposemin(-thickimsegreg,max(fg,[],3)));
catch ME
    rethrow(ME)
end

Lcell = cellbw.*repmat(Lreguse,[1 1 size(cellim,3)]);
Lcell2D = max(Lcell,[],3);
thickim = sum(cellbw,3);

[x,y] = makegrid2(Lcell2D);

interior = imerode(Lcell>0,cat(3,[0,0,0;0,1,0;0,0,0],[0,1,0;1,1,1;0,1,0],[0,0,0;0,1,0;0,0,0]));
cellsurf = (Lcell>0) & ~interior;

Lint = interior.*Lcell;
Lsurf = cellsurf.*Lcell;

Lbound2 = max(Lcell,[],3) - max(Lint,[],3);
% Lbound2 should have the 2D boundaries of each cell

for ii = 1:max(Lreguse(:))
    cellprops(ii).vol = nnz(Lcell==ii);
    cellprops(ii).cellxy(1,1) = sum(x(Lcell2D==ii).*thickim(Lcell2D==ii))/cellprops(ii).vol;
    cellprops(ii).cellxy(1,2) = sum(y(Lcell2D==ii).*thickim(Lcell2D==ii))/cellprops(ii).vol;

    % to estimate GFP intensity, want to avoid edge effects as far as
    % possible, so calculate the median of the interior.  Is this too
    % slow?

    cellprops(ii).gfpint = median(cellim(Lint==ii));

    % what else?  Some measure of shape elongation? Surface to volume
    % ratio?

    cellprops(ii).surfvol = nnz(Lsurf==ii);
    [cellprops(ii).v,cellprops(ii).d] = eigvect(Lcell2D==ii,cellprops(ii).cellxy);
    cellprops(ii).perim2 = nnz(Lbound2==ii); % 2D perimeter
    
    cellprops(ii).slices = squeeze(sum(sum(Lcell==ii,1),2));

end
 
%  [];
 
 
 
 
 

