function [cellprops,Lcell] = segcell3new(cellim,fg,nhoodsiz,val3,zval)

if nargin<5 || isempty(zval)
    zval = 1.5;
end
if nargin<4 || isempty(val3)
    val3 = 0.8;
end
if nargin<3 || isempty(nhoodsiz)
    nhoodsiz = [round(size(cellim,1)/5), round(size(cellim,2)/5), size(cellim,3)];
end


if nargin<2
    fg = [];
end

if ~isempty(fg)
    if size(fg,2)==2 || size(fg,2)==3
        fgxy = round(fg);
        fg = zeros(size(cellim,1),size(cellim,2));
        for ii = 1:size(fgxy,1)
            fg(fgxy(ii,1),fgxy(ii,2)) = ii;
        end
    end
end

if size(fg,3)>1
    fg = max(fg,[],3);
end

% first perform the thresholding, then if we have foreground markers, use
% these to divide up the cells.

% slice normalisation
im3 = cellim - repmat(mean(mean(cellim,1),2),[size(cellim,1), size(cellim,2), 1]);

im2d = sort(im3,3,'ascend');
ind = ceil(0.8*size(im2d,3));
if ind>1
    im2d = mean(im2d(:,:,(ind-1):ind),3);
end

bw = adthreshstd(im2d,nhoodsiz,zval);
bw2 = imopen(bw,true(4));

bgval = mean(im3(~repmat(bw2,[1,1,size(cellim,3)])));

mask3 = (im3-bgval)>repmat(val3*(max(im3,[],3)-bgval),[1,1,size(cellim,3)]);

bw3 = mask3 & repmat(bw2,[1,1,size(cellim,3)]);

bw4 = adimfilter(bw3,[3,3,3])>3/9;

if isempty(fg)
    Lcell = bwlabeln(bw4);
else
    % this will crash! Best to implement the 2D watershedding in
    % segcell3full.
    L1 = watershed(imimposemin(-bw4, fg>0));
    Lcell = L1;
    if max(fg(:))>1
        % renumber the labels to match the fg values
        for ii = 1:max(fg(:))
            fginds = find(fg(:)==ii);
    
            for jj = 1:numel(fginds)
                Lcell(L1==L1(fginds(jj))) = ii;
            end
        end
        
    end
end

Lcell2D = max(Lcell,[],3);

% now find the same properties as in segcell3full
interior = imerode(Lcell>0,cat(3,[0,0,0;0,1,0;0,0,0],[0,1,0;1,1,1;0,1,0],[0,0,0;0,1,0;0,0,0]));
cellsurf = (Lcell>0) & ~interior;

Lint = interior.*Lcell;
Lsurf = cellsurf.*Lcell;

Lbound2 = max(Lcell,[],3) - max(Lint,[],3);
% Lbound2 should have the 2D boundaries of each cell

thickim = sum(Lcell>0,3);

[x,y] = makegrid2(Lcell2D);

for ii = 1:max(Lcell(:))
    cellprops(ii).vol = nnz(Lcell==ii);
    cellprops(ii).cellxy(1,1) = sum(x(Lcell2D==ii).*thickim(Lcell2D==ii))/cellprops(ii).vol;
    cellprops(ii).cellxy(1,2) = sum(y(Lcell2D==ii).*thickim(Lcell2D==ii))/cellprops(ii).vol;

    % to estimate GFP intensity, want to avoid edge effects as far as
    % possible, so calculate the median of the interior.  Is this too
    % slow?

    cellprops(ii).gfpint = median(cellim(Lint==ii));

    % what else?  Some measure of shape elongation? Surface to volume
    % ratio?

    cellprops(ii).surfvol = nnz(Lsurf==ii);
    [cellprops(ii).v,cellprops(ii).d] = eigvect(Lcell2D==ii,cellprops(ii).cellxy);
    cellprops(ii).perim2 = nnz(Lbound2==ii); % 2D perimeter
    
    cellprops(ii).slices = squeeze(sum(sum(Lcell==ii,1),2));

end
 





