function [cellprops,Lcell] = segcell3nuc(cellim,Lnuc,threshfact)

% segment the cells using nuclei as foreground markers, and the nuclear
% mask to tell us about the GFP intensity in each cell.

% we can use the nuclear mask at any stage as a tool to determine a
% threshold to be used:
% - first attempt: median filter then nuclear threshold

if nargin<3 || isempty(threshfact)
    threshfact = 0.7;
end

% subtract off mean of each slice
mz = mean(mean(cellim,1),2);
im = cellim - repmat(mz,[size(cellim,1),size(cellim,2),1]);

medim = ordfiltz(adordfilt(im,discel(6,1)),3,0.3);

bglevel = mean(medim(~(Lnuc>0))); %dilating the nuclear mask first would get less interference from cells

% need a different threshold for each cell

cellbw = false(size(cellim));

for ii = 1:max(Lnuc(:))
    locallevel = mean(medim(Lnuc==ii));
    localthresh = bglevel + threshfact*(locallevel-bglevel);
    
    % find the region above the threshold which also belongs to the nuclei
    bw = medim>localthresh;
    % only keep regions which overlap with the nuclear mask
    Ltemp = bwlabeln(bw);
    keepreg = unique(Ltemp(Lnuc==ii));
    keepreg(keepreg==0) = [];
    
    for jj = 1:numel(keepreg)
        cellbw(Ltemp==keepreg(jj)) = true;
    end
    
end

% cellbw then needs segmenting into individual cells, use same procedure as
% segcell3full

Lreguse = watershed(imimposemin(-sum(cellbw,3),max(Lnuc>0,[],3)));
Lcell = cellbw.*repmat(Lreguse,[1 1 size(cellim,3)]);
Lcell2D = max(Lcell,[],3);
thickim = sum(cellbw,3);

[x,y] = makegrid2(Lcell2D);

interior = imerode(Lcell>0,cat(3,[0,0,0;0,1,0;0,0,0],[0,1,0;1,1,1;0,1,0],[0,0,0;0,1,0;0,0,0]));
cellsurf = (Lcell>0) & ~interior;

Lint = interior.*Lcell;
Lsurf = cellsurf.*Lcell;

Lbound2 = max(Lcell,[],3) - max(Lint,[],3);
% Lbound2 should have the 2D boundaries of each cell

for ii = 1:max(Lreguse(:))
    cellprops(ii).vol = nnz(Lcell==ii);
    cellprops(ii).cellxy(1,1) = sum(x(Lcell2D==ii).*thickim(Lcell2D==ii))/cellprops(ii).vol;
    cellprops(ii).cellxy(1,2) = sum(y(Lcell2D==ii).*thickim(Lcell2D==ii))/cellprops(ii).vol;

    % to estimate GFP intensity, want to avoid edge effects as far as
    % possible, so calculate the median of the interior.  Is this too
    % slow?

    cellprops(ii).gfpint = median(cellim(Lint==ii));

    % what else?  Some measure of shape elongation? Surface to volume
    % ratio?

    cellprops(ii).surfvol = nnz(Lsurf==ii);
    [cellprops(ii).v,cellprops(ii).d] = eigvect(Lcell2D==ii,cellprops(ii).cellxy);
    cellprops(ii).perim2 = nnz(Lbound2==ii); % 2D perimeter
    
    cellprops(ii).slices = squeeze(sum(sum(Lcell==ii,1),2));

end

