function [cellprops,Lcell] = segcell4(im,Lnuc,siz)

% updated version of cell segmentation algorithm, to work better with
% deconvolved images.

%%%%%%%%%
% 2D NUCLEAR LABEL MATRIX REQUIRED!!

if nargin<3 || isempty(siz)
    siz = 20;
end

% mz = mean(mean(im,1),2);
% im = im - repmat(mz,[size(im,1),size(im,2),1]);

ordim = mean(im,3);

imoc = iobrcbr(ordim,discel(ceil(siz/4),1,2*ceil(siz/4)-1));
gradim = sobel2(imoc);

% fg = bwmorph(max(Lnuc,[],3)>0,'shrink',Inf);
% breaks down when two nuclei are touching or overlapping in 2D
fg = max(Lnuc,[],3)>0;


bg = watershed(-imoc)==0;
L = watershed(imimposemin(gradim,fg|bg));

% merge possibly multinuclear cells for the purposes of finding the top and
% bottom.

bgvals = unique(L(bg));
for ii = 1:numel(bgvals)
    L(L==bgvals(ii)) = 0;
end

L = rescalelabels(L);

lvals = unique(L(:));

L2 = L; % copy of L.

nuccellmatch = false(max(Lnuc(:)),max(L(:))); %should be square?

for ii = 1:max(Lnuc(:))
    celllabel = unique(L(Lnuc==ii));
    celllabel(celllabel==0) = [];
    nuccellmatch(ii,celllabel) = true;
end

% how likely is a triple overlapping cell?

usednuc = false(max(Lnuc(:)),1);
usedcell = false(max(L(:)),1);
bw = false(size(im));
thickim = zeros(size(im,1),size(im,2));
Lcell2 = zeros(size(thickim));

for ii = 1:max(L(:))
    tempcell = false(max(L(:))+1,1);
    tempnuc = false(max(Lnuc(:))+1,1);
    % see which nuclei fall within this cell, and then which cells fall
    % within those nuclei
    if ~usedcell(ii)
        nucs = find(nuccellmatch(:,ii));
        cells = [];
        for jj = 1:numel(nucs)
            cells = [cells,find(nuccellmatch(nucs(jj),:))];
        end
        
        % could iterate this until neither nucs nor cells changes if we
        % need to
        
        cells = unique(cells);
        for jj = 1:numel(cells)
            nucs = [nucs;find(nuccellmatch(:,cells(jj)))];
        end
        nucs = unique(nucs);
        % group these cells together for the purposes of z-segmentation, then
        % rewatershed based on the thickness.
        
        tempcell(cells + 1) = true;
        theseL = imclose(tempcell(L+1),true(3,3));
        
        tempnuc(nucs + 1) = true;
        theseN = tempnuc(Lnuc+1);
        
        slim = reshape(im(repmat(theseL,[1 1 size(im,3)])),[nnz(theseL),size(im,3)]);
        mi = admean(im(theseN));
        si = adstd(im(theseN));
        smslim = iobrcbr(slim,true(1,5));
        
        slimbw = smslim>(mi-si);
        
        bw(repmat(theseL,[1,1,size(im,3)])) = slimbw(:);
        
        thickim(theseL) = sum(slimbw,2);
        
        
        
%         keyboard
%         ii
        if numel(cells)>1 || numel(nucs)>1
            % now need to resegment the cells if we can
            thickim0 = zeros(size(thickim));
            thickim0(theseL) = sum(slimbw,2);
            L0 = watershed(imimposemin(-thickim0,theseN));
            L0(~theseL) = 0;
            
            if max(L0(:))~=numel(nucs)
                % touching nuclei have messed up the watershed
                % segmentation, let's just segment using the distance
                % transform instead.
                distim = zeros(size(L0,1),size(L0,2),numel(nucs));
                for jj = 1:numel(nucs)
                    distim(:,:,jj) = bwdist(Lnuc==nucs(jj));
                end
                mindist = min(distim,[],3);
                L0 = zeros(size(Lnuc));
                for jj = 1:numel(nucs)
                    L0(distim(:,:,jj)==mindist) = jj;
                end
                L0(~theseL) = 0;
            end
            
            % reassign the labels to match the numbers of the nuclei
            for jj = 1:numel(nucs)
                thiscell = unique(L0(Lnuc==nucs(jj)));
                thiscell(thiscell==0) = [];
                % common error when thiscell has more than one element!
                % what's causing this?  Seems to be if the nuclear label is
                % fragmented..
                try
                    Lcell2(L0==thiscell) = nucs(jj);
                catch ME
                    rethrow(ME)
                end
            end
        else
            % theseL already contains the footprint of the cell
            % assign the label matrix based on the nuclei number
            try
                Lcell2(theseL) = nucs;
            catch ME
                rethrow(ME)
            end
        end
    end
end

Lcell = repmat(Lcell2,[1,1,size(im,3)]).*bw;
[x,y] = makegrid2(Lcell2);

interior = imerode(Lcell>0,cat(3,[0,0,0;0,1,0;0,0,0],[0,1,0;1,1,1;0,1,0],[0,0,0;0,1,0;0,0,0]));
cellsurf = (Lcell>0) & ~interior;

Lint = interior.*Lcell;
Lsurf = cellsurf.*Lcell;

Lcell2 = max(Lcell,[],3);
Lbound2 = Lcell2 - max(Lint,[],3); % could set all non-zero Lint to zero instead of doing arithmetic
% Lbound2 should have the 2D boundaries of each cell

% instead of removing cells with anomalous volumes (since the nuclei have
% been segmented successfully), set the cell boundary to be the nuclear
% cylinder, and flag the cell as having poor segmentation.

for ii = 1:max(Lcell2(:))
    cellprops(ii).vol = nnz(Lcell==ii);
    if cellprops(ii).vol==0
        cellprops(ii).badseg = 1;
        cellprops(ii).cellxy(1,1) = sum(x(Lnuc==ii))/nnz(Lnuc==ii);
        cellprops(ii).cellxy(1,2) = sum(y(Lnuc==ii))/nnz(Lnuc==ii);
        cellprops(ii).gfpint = median(im(repmat(Lnuc==ii,[1,1,size(im,3)])));
        % other measures are irrelevant if the segmentation wasn't
        % successful
        
        % add a cylindrical shape to Lcell to avoid messing up the spot
        % detection
        Lcell(repmat(Lnuc==ii,[1,1,size(im,3)])) = ii;
    else
        cellprops(ii).badseg = 0;
        cellprops(ii).cellxy(1,1) = sum(x(Lcell2==ii).*thickim(Lcell2==ii))/cellprops(ii).vol;
        cellprops(ii).cellxy(1,2) = sum(y(Lcell2==ii).*thickim(Lcell2==ii))/cellprops(ii).vol;
        
        % measure cell area
        cellprops(ii).area = nnz(Lcell2==ii);
        
        % to estimate GFP intensity, want to avoid edge effects as far as
        % possible, so calculate the median of the interior.  Is this too
        % slow?

        cellprops(ii).gfpint = median(im(Lint==ii));

        % what else?  Some measure of shape elongation? Surface to volume
        % ratio?

        cellprops(ii).surfvol = nnz(Lsurf==ii);
        [cellprops(ii).v,cellprops(ii).d] = eigvect(Lcell2==ii,cellprops(ii).cellxy);
        cellprops(ii).perim2 = nnz(Lbound2==ii); % 2D perimeter
        
        cellprops(ii).slices = squeeze(sum(sum(Lcell==ii,1),2));
    end
end


