function [cellprops,Lcell,Lcell2] = segcell5(im,Lnuc,siz,bgthr)

% updated version of cell segmentation algorithm, to work better with
% deconvolved images.

% changed in version 5:
% - don't need the mean subtraction z-normalisation at the start for
%   deconvolved images, removed.
% - background segmentation changed from watershedding to thresholding
%   followed by morphological closing and skeletonization.

% POTENTIAL FURTHER CHANGES
% - use some sort of edge preserving smoothing filter (eg bilateral)
%   instead of opening and closing by reconstruction.

%%%%%%%%%
% 2D NUCLEAR LABEL MATRIX REQUIRED!!
if nargin<4 || isempty(bgthr)
    bgthr = 'auto';
end

if nargin<3 || isempty(siz)
    siz = 20;
end

% mz = mean(mean(im,1),2);
% im = im - repmat(mz,[size(im,1),size(im,2),1]);

if ~isempty(Lnuc) && max(Lnuc(:))==0
    % no nuclei detected
    cellprops = [];
    Lcell = [];
    Lcell2 = [];
    return
end

ordim = mean(im,3);
if ischar(bgthr) && strcmpi(bgthr,'auto')
    % find the background threshold as the first peak in the intensity
    % histogram
    [n,x] = hist(ordim(:),256);
    bgthr = nearestmaxfit1(x,n,0,3); % quickly implement here rather than calling another function?
end

bg = bwmorph(imclose(ordim<bgthr,discel(4,1)),'skel',Inf);

imoc = iobrcbr(ordim,discel(ceil(siz/4),1,2*ceil(siz/4)-1));
gradim = sobel2(imoc);

% fg = bwmorph(max(Lnuc,[],3)>0,'shrink',Inf);

% let's first check for problems with the nuclear mask
% how much will this slow things down by?

if ~isempty(Lnuc)

    Ln2 = max(Lnuc,[],3);

    % could do this first loop in area order if we want, to preserve smaller
    % regions at the expense of larger ones.
    for ii = 1:max(Ln2(:))
        Ln2(imdilate(Ln2==ii,true(3)) & ~(Ln2==ii))=0;
    end

    for ii = 1:max(Ln2(:))
        [tL,tN] = bwlabeln(Ln2==ii);
        if tN>1
            fragstats = regionprops(tL,'area');
            keepind = find([fragstats.Area]==max([fragstats.Area]),1,'first');
            Ln2(Ln2==ii)=0;
            Ln2(tL==keepind)=ii;
        end
    end

    % breaks down when two nuclei are touching or overlapping in 2D
    fg = Ln2>0;
else
    % come up with a fake Lnuc by estimating where the centres of cells are
    fi = dog3split(0.25*siz,ceil(2*siz));
    fim = gauss2filt(log(mean(im,3)),fi);
    fg = imregionalmax(fim);
    
    Lnuc = bwlabeln(fg);
    
    
%     keyboard
    
end

L = watershed(imimposemin(gradim,fg|bg));

% merge possibly multinuclear cells for the purposes of finding the top and
% bottom.

bgvals = unique(L(bg));
for ii = 1:numel(bgvals)
    L(L==bgvals(ii)) = 0;
end

L = rescalelabels(L);

% % lvals = unique(L(:));

% % L2 = L; % copy of L.

nuccellmatch = false(max(Lnuc(:)),max(L(:))); %should be square?

for ii = 1:max(Lnuc(:))
    celllabel = unique(L(Ln2==ii));
    celllabel(celllabel==0) = [];
    nuccellmatch(ii,celllabel) = true;
end

% how likely is a triple overlapping cell?

% % usednuc = false(max(Lnuc(:)),1);
usedcell = false(max(L(:)),1);
bw = false(size(im));
thickim = zeros(size(im,1),size(im,2));
Lcell2 = zeros(size(thickim));

nucl = im(:,:,1);
bgl = admean(nucl(bg));

nucu = im(:,:,end);
bgu = admean(nucu(bg));

for ii = 1:max(L(:))
    tempcell = false(max(L(:))+1,1);
    tempnuc = false(max(Lnuc(:))+1,1);
    % see which nuclei fall within this cell, and then which cells fall
    % within those nuclei
    if ~usedcell(ii)
        nucs = find(nuccellmatch(:,ii));
        cells = [];
        for jj = 1:numel(nucs)
            cells = [cells,find(nuccellmatch(nucs(jj),:))];
        end
        
        % could iterate this until neither nucs nor cells changes if we
        % need to
        
        cells = unique(cells);
        for jj = 1:numel(cells)
            nucs = [nucs;find(nuccellmatch(:,cells(jj)))];
        end
        nucs = unique(nucs);
        % group these cells together for the purposes of z-segmentation, then
        % rewatershed based on the thickness.
        
        tempcell(cells + 1) = true;
        theseL = imclose(tempcell(L+1),true(3,3));
        
        tempnuc(nucs + 1) = true;
        theseN = tempnuc(Lnuc+1);
        
        slim = reshape(im(repmat(theseL,[1 1 size(im,3)])),[nnz(theseL),size(im,3)]);
% %         mi = admean(im(theseN));
% %         si = adstd(im(theseN));
% %         smslim = iobrcbr(slim,true(1,5));
% %         
% %         slimbw = smslim>(mi-si);
% %         
% %         bw(repmat(theseL,[1,1,size(im,3)])) = slimbw(:);
% %         
% %         thickim(theseL) = sum(slimbw,2);
% %         
        
        
                smslim = adordfilt(slim,true(3,5),0.5);
                % WHICH IS BETTER?

                slp = [bgl*ones(size(slim,1),1),smslim,bgu*ones(size(slim,1),1)];

                slimbw = zeros(size(smslim));
                % find the maximal gradients either side of the maxima
                zz = 1:size(smslim,2);
                for kk = 1:size(smslim,1)
                    maxind = find(smslim(kk,:)==max(smslim(kk,:)),1,'first');
                    sgrad = slp(kk,3:end) - slp(kk,1:(end-2));
                    if maxind>1
                        lval = max(sgrad(zz<maxind));
                        lind = find(zz<maxind & sgrad==lval,1,'first');
                    else
                        lind = 1;
                    end

                    if maxind<size(smslim,2)
                        uval = min(sgrad(zz>maxind));
                        uind = find(zz>maxind & sgrad==uval,1,'last');
                    else
                        uind = size(smslim,2);
                    end

%                     slimbw(kk,lind:uind) = ii;
                    slimbw(kk,lind:uind) = 1;
                end

            %     slimbw = ii*(smslim>(nucv-0.8*nucstd));

%                 L(repmat(thisN,[1,1,size(im,3)])) = slimbw(:);

                bw(repmat(theseL,[1,1,size(im,3)])) = slimbw(:);
                thickim(theseL) = sum(slimbw,2);
    
        
        
%         keyboard
%         ii
        if numel(cells)>1 || numel(nucs)>1
            % now need to resegment the cells if we can
            thickim0 = zeros(size(thickim));
            thickim0(theseL) = sum(slimbw,2);
            L0 = watershed(imimposemin(-thickim0,theseN));
            L0(~theseL) = 0;
            
            if max(L0(:))~=numel(nucs)
                % touching nuclei have messed up the watershed
                % segmentation, let's just segment using the distance
                % transform instead.
                distim = zeros(size(L0,1),size(L0,2),numel(nucs));
                for jj = 1:numel(nucs)
                    distim(:,:,jj) = bwdist(Lnuc==nucs(jj));
                end
                mindist = min(distim,[],3);
                L0 = zeros(size(Lnuc));
                for jj = 1:numel(nucs)
                    L0(distim(:,:,jj)==mindist) = jj;
                end
                L0(~theseL) = 0;
            end
            
            % reassign the labels to match the numbers of the nuclei
            for jj = 1:numel(nucs)
                thiscell = unique(L0(Lnuc==nucs(jj)));
                thiscell(thiscell==0) = [];
                % common error when thiscell has more than one element!
                % what's causing this?  If one of the nuclear labels is
                % fragmented and it also touches another of the nuclei,
                % this gets messed up
                try
                    Lcell2(L0==thiscell) = nucs(jj);
                catch ME
                    rethrow(ME)
                end
            end
        else
            % theseL already contains the footprint of the cell
            % assign the label matrix based on the nuclei number
            try
                Lcell2(theseL) = nucs;
            catch ME
                rethrow(ME)
            end
        end
    end
end

Lcell = repmat(Lcell2,[1,1,size(im,3)]).*bw;
[x,y] = makegrid2(Lcell2);

interior = imerode(Lcell>0,cat(3,[0,0,0;0,1,0;0,0,0],[0,1,0;1,1,1;0,1,0],[0,0,0;0,1,0;0,0,0]));
cellsurf = (Lcell>0) & ~interior;

Lint = interior.*Lcell;
Lsurf = cellsurf.*Lcell;

% Lcell2 = max(Lcell,[],3);
% Lbound2 = Lcell2 - max(Lint,[],3); % could set all non-zero Lint to zero instead of doing arithmetic


Lbound2 = Lcell2 - imerode(Lcell2,[0,1,0;1,1,1;0,1,0]);
% Lbound2 should have the 2D boundaries of each cell

% instead of removing cells with anomalous volumes (since the nuclei have
% been segmented successfully), set the cell boundary to be the nuclear
% cylinder, and flag the cell as having poor segmentation.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OTHER THINGS WE CAN MEASURE

% - number of other cells which are touching the boundary of the cell
% - fraction of cell boundary which is in contact with another cell

for ii = 1:max(Lcell2(:))
    cellprops(ii).vol = nnz(Lcell==ii);
    if cellprops(ii).vol==0
        cellprops(ii).badseg = 1;
        cellprops(ii).cellxy(1,1) = sum(x(Ln2==ii))/nnz(Ln2==ii);
        cellprops(ii).cellxy(1,2) = sum(y(Ln2==ii))/nnz(Ln2==ii);
        cellprops(ii).gfpint = median(im(repmat(Ln2==ii,[1,1,size(im,3)])));
        % other measures are irrelevant if the segmentation wasn't
        % successful
        
        % add a cylindrical shape to Lcell to avoid messing up the spot
        % detection
        Lcell(repmat(Ln2==ii,[1,1,size(im,3)])) = ii;
    else
        cellprops(ii).badseg = 0;
        cellprops(ii).cellxy(1,1) = sum(x(Lcell2==ii).*thickim(Lcell2==ii))/cellprops(ii).vol;
        cellprops(ii).cellxy(1,2) = sum(y(Lcell2==ii).*thickim(Lcell2==ii))/cellprops(ii).vol;
        
        % measure cell area
        cellprops(ii).area = nnz(Lcell2==ii);
        
        % to estimate GFP intensity, want to avoid edge effects as far as
        % possible, so calculate the median of the interior.  Is this too
        % slow?

        cellprops(ii).gfpint = median(im(Lint==ii));

        % what else?  Some measure of shape elongation? Surface to volume
        % ratio?
        
        cellprops(ii).surfvol = nnz(Lsurf==ii);
        [cellprops(ii).v,cellprops(ii).d] = eigvect(Lcell2==ii,cellprops(ii).cellxy);
        cellprops(ii).perim2 = nnz(Lbound2==ii); % 2D perimeter
        
        % Cell neighbour information
        bounddata = Lcell2(imdilate(Lcell2==ii,discel(2.5,1,7)) & ~imdilate(Lcell2==ii,true(3,3)));
        bounddata(bounddata==ii) = []; % shouldn't need this
        tn = unique(bounddata);
        tn(tn==0) = [];
        cellprops(ii).touching = tn;
        cellprops(ii).touchingfrac = nnz(bounddata)/numel(bounddata);
        
        
        cellprops(ii).slices = squeeze(sum(sum(Lcell==ii,1),2));
    end
end


