function [Lcell,Lnuc] = segcellfast(cellim,nucbw,nhood)
% need to find nuclei, or some other marker inside the cell, first.

% if nargin<4 || isempty(figs)
%     figs = 1;
% end
if nargin<3 || isempty(nhood)
    nhood = discel(8,1);
end
if numel(nhood)==1
    nhood = discel(nhood,1);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fast version of cell segmenting, using adaptive thresholding
%
% This could probably be generalised into 3D to improve the segmentation of
% out-of-focus slices


% imoc = iobrcbr(cellim,nhood);
% 
% gradim = sobel2(imoc);
% 
% bg = watershed(-imoc)==0;


if nargin<2 || isempty(nucbw)
%     shownuc = 0;
    % need to come up with our own foreground markers
%     gradoc = iobrcbr(gradim,nhood);
%     fg = ordfilt2(imclearborder(gradoc==0),ceil(nnz(discel(5,1))/2),discel(5,1));
    % remove parts of fg touching edge of image
    
    % NEW METHOD
    % - adaptive thresholding and standard deviation filter of result -
    % selects plateaus of bright areas.
    % - Then shrink down to points using bwmorph
    
    % IMPORTANT!
    % If speed is still an issue, the above method could be used as a crude
    % segmentation method, avoiding the need for gradient calculation and
    % watershedding, and even opening and closing by reconstruction, which
    % is the most time-consuming part.
    
    % size of adaptive region ~ 4*nhoodrad == 2*(size(nhood,1)-1)
    
    bw1 = adthresh(cellim,[round(3.6*(size(nhood,1)-1)),round(3.6*(size(nhood,2)-1)),size(cellim,3)],0);
    bw2 = stdfilt(bw1,discel(3,1))<0.05; % relate the size of this neighbourhood to the size in the input?
    
    % possibly erode before/instead of shrinking to make sure single noisy
    % pixels are removed
%     fg = bwmorph(bw1 & bw2,'shrink',Inf);
    Lcell = bwlabeln(bw1 & bw2);
    
else
%     shownuc = 1;

    if ~isa(nucbw,'logical')
        %nucbw is the unsegmented nucleus fluorescence image, or a list of
        %coordinates
        if (size(nucbw,2)==2) || (size(nucbw,1)==2)
            if size(nucbw,2)>size(nucbw,1)
                nucbw = nucbw';
            end
            % now do something about it!
            % MORE HERE!
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        end

        [xdim,ydim] = size(cellim);

        [x,y] = makegrid2(zeros(4*sigma + 1));
        template = exp(-((x-(2*sigma+1)).^2 + (y-(2*sigma+1)).^2)/sigma^2);

        C = normxcorr2(template,padarray(im,[(2*sigma+1) (2*sigma+1)],0));
        % might be necessary to pad the array a little more (will then have to correct
        % the cropping below too)

        C2 = C((2+4*sigma):(xdim+1+4*sigma),(2+4*sigma):(ydim+1+4*sigma));

        % foreground for the watershed segmentation
        fg = C2>thresh;
        if nargout>1
            % we also need to segment the nucleus and output the labelled
            % image
            nhood2 = discel(0.8*sigma,1);
            gradim2 = sobel2(iobrcbr(im,nhood2));
            % need a background, otherwise the faintest nucleus will disappear
            bg = watershed(-fg)==0;
            Lnuc = watershed(imimposemin(gradim2,fg|bg));
            % ought to have some check on the size and shape of the nuclei, but
            % should that be done here or outside?
        end
    else
        fg = nucbw;
    end

    bw1 = adaptivethreshold(cellim,round(3.6*(size(nhood,1)-1)),0);
    bw2 = stdfilt(bw1,discel(3,1))<0.05; % relate the size of this neighbourhood to the size in the input?
    
    % possibly erode before/instead of shrinking to make sure single noisy
    % pixels are removed
%     fg = bwmorph(bw1 & bw2,'shrink',Inf);
    Lcell = bwlabeln(bw1 & bw2);
    
    % only keep regions which overlap with a nucleus marker
    count = 1;
    for ii = 1:max(Lcell(:))
        if nnz(fg & (Lcell==ii))
            Lcell(Lcell==ii) = count;
            count = count + 1;
        else
            Lcell(Lcell==ii) = 0;
        end
    end
    
end


% Lcell = watershed(imimposemin(gradim,fg|bg));
% 
% if figs
%     figure
%     imshow(label2rgb(Lcell,'jet','w','shuffle'))
% 
%     figure
%     if shownuc
%         imgray(normalise(cellim)+nucbw+(Lcell==0))
%     else
%         imgray(normalise(cellim)+(Lcell==0))
%     end
% end

