function [pos,L,nucstats] = segnuc(im,sigma,thresh)

% find positions and segment the nuclei

% positions first
% since they are just bright spots, try a Gaussian template (top hat would
% probably also work well)

if nargin<3 || isempty(thresh)
    thresh = 0.7;
    % could maybe lower this to find noisy nuclei
end
if nargin<2 || isempty(sigma)
    sigma = 5;
end

[xdim,ydim] = size(im);

[x,y] = makegrid2(zeros(4*sigma + 1));
template = exp(-((x-(2*sigma+1)).^2 + (y-(2*sigma+1)).^2)/sigma^2);

C = normxcorr2(template,padarray(im,[(2*sigma+1) (2*sigma+1)],'symmetric'));
% might be necessary to pad the array a little more (will then have to correct
% the cropping below too)

C2 = C((2+4*sigma):(xdim+1+4*sigma),(2+4*sigma):(ydim+1+4*sigma));

% foreground for the watershed segmentation
fg = C2>thresh;

%%%%%%%%%
% get positions

% no need for subpixel accuracy yet
L1 = bwlabeln(fg);
stats = regionprops(L1,'Centroid');
pos = field2val(stats,'Centroid');



%%%%%%%%%%%%%%%%%%
% if a second output is required, use the positions to segment the nuclei
if nargout>1
    nhood = discel(0.8*sigma,1);
    gradim = sobel2(iobrcbr(im,nhood));
    % need a background, otherwise the faintest nuclei may disappear
    bg = watershed(-fg)==0;
    L = watershed(imimposemin(gradim,fg|bg));
    % ought to have some check on the size and shape of the nuclei, but
    % should that be done here or outside? Try here
    
    % remove any objects touching the edge of the image first
    L = bwlabeln(imclearborder(L>0));
    
    Ler = imerode(L,[0,1,0;1,1,1;0,1,0]);
    
    nucstats = regionprops(L,'Area','PixelIdxList','Centroid','BoundingBox');
    for ii = 1:length(nucstats)
        % also get some measure of the intensity for comparison
        nucstats(ii).sumint = sum(im(nucstats(ii).PixelIdxList));
        nucstats(ii).meanint = nucstats(ii).sumint/nucstats(ii).Area;
        nucstats(ii).edgepix = findn((L==ii) - (Ler==ii));
        nucstats(ii).xyz = nucstats(ii).Centroid([2 1]);
    end
    % update pos
    pos = field2val(nucstats,'Centroid');
end

% flip pos so x and y are in the right order
if size(pos,2)>1
    pos = pos(:,[2 1]);
end

