function [xyz,L,nucstats] = segnuc4(im,sigma,decon,stdfact,texprops)


% 3-dimensional nuclear segmentation
% as well as giving a foreground for the cell segmentation, the nuclear
% segmentation will also give us information on intranuclear spot
% localisation and nuclear volume, etc.
% since the spots aren't going to be known at this stage, we need some way
% of recording the nucleus shape and size, perhaps simply as a list of
% boundary coordinates.  In this form, it will give us an easy way of
% calculating the shortest distance to the edge of the nucleus, including
% z-scaling.

% at the moment, two nuclei can't be directly on top of each other, since
% for speed purposes the foreground finding is still done in 2D.  This
% could easily be changed if needed.

if nargin<5 || isempty(texprops)
    texprops = 0;
end
if nargin<4 || isempty(stdfact)
    stdfact = 0.6;
end

if nargin<3 || isempty(decon)
    decon = 0;
end

if nargin<2 || isempty(sigma)
    sigma = 5;
end

[xdim,ydim,zdim] = size(im);

im2d = mean(im,3); % max is an alternative here, or the 3/4 order statistic?

[x,y] = makegrid2(zeros(4*sigma + 1));
template = exp(-((x-(2*sigma+1)).^2 + (y-(2*sigma+1)).^2)/sigma^2);

C = normxcorr2(template,padarray(im2d,[(2*sigma+1) (2*sigma+1)],'symmetric'));
% might be necessary to pad the array a little more (will then have to correct
% the cropping below too)

C2 = C((2+4*sigma):(xdim+1+4*sigma),(2+4*sigma):(ydim+1+4*sigma));

C2(C2<0.6) = 0;

fguse = imregionalmax(C2);

% % % if ~decon
% % %     % now need to segment in 3D
% % %     nucbw = imopen(adthreshstd(im,[30 30 30],stdfact),ones(5,5));
% % %     % discel would probably be better than ones(5,5), but is this slower?
% % % 
% % %     % it's this closing operation which tends to join together the nuclei of
% % %     % multinuclear cells.
% % %     nucbw = imclose(nucbw,ones(5,5,2));
% % % else
% % %     nucbw = imopen(adthreshstd(im,[30,30,30],1),ones(3,3,3));
% % %     nucbw = imclose(nucbw,ones(3,3,3));
% % %     % seems to keep the precise boundary better when we have deconvolved
% % %     % images
% % % end
% % % 
% % % fguse = fg .* max(nucbw,[],3);

xyinds = findn(fguse);
xyz = zeros(size(xyinds,1),3);
fg3d = false(size(im));

% we could use bwlabeln first to label the segmented nuclei, then check
% that none of them are joined together, and if they are, run the
% watershedding algorithm.

% % % testL = bwlabeln(nucbw);

for ii = 1:size(xyinds,1)
    smoothint = adsmooth(squeeze(im(xyinds(ii,1),xyinds(ii,2),:)),5);
    xyz(ii,:) = [xyinds(ii,:), find(smoothint==max(smoothint),1,'first')];
    fg3d(xyz(ii,1),xyz(ii,2),xyz(ii,3)) = true;
end

% test2d = repmat(imclearborder(max(testL>0,[],3)),[1 1 size(testL,3)]);
% % remove edge objects
% testL = testL.*test2d;

% might be useful to save the edge objects somewhere to help with
% segmentation of overlapping objects.

% As in the cell segmentation, take the nuclear intensity at our measured
% centre and use this as a threshold level.  THIS DOESN'T WORK VERY WELL!
% ESPECIALLY FOR FAINT NUCLEI

imoc = iobrcbr(im2d,discel(4,1));
bg = watershed(-imoc)==0;
bgv = mean(im2d(bg));
localthresh = -Inf*ones(size(im,1),size(im,2));

for ii = 1:size(xyz,1)
    nucv = mean(mean(mean(im(max(1,xyz(ii,1)-2):min(xdim,xyz(ii,1)+2),...
        max(1,xyz(ii,2)-2):min(ydim,xyz(ii,2)+2),...
        max(1,xyz(ii,3)-2):min(zdim,xyz(ii,3)+2)))));
    localthresh(xyz(ii,1),xyz(ii,2)) = bgv+stdfact*(nucv-bgv);
end

localthresh = repmat(imdilate(localthresh,discel(2*sigma,1)),[1,1,size(im,3)]);
localthresh(isinf(localthresh)) = Inf;

bw = imopen(imclose(im>localthresh,ones(5,5)),ones(3,3));

testL = bwlabeln(bw);

% now quickly check whether there's a single fg marker in each region.
maxnum = 0;

for ii = 1:max(testL(:))
    if nnz(testL==ii)<120
        testL(testL==ii) = 0;
    else
        numnuc = sum(fg3d(testL==ii));
%             if numnuc>1
%                 ii
% % %                 keyboard
%             end
    
        % remove objects on the edge and objects below a specified size
        % threshold here too
    
        if numnuc==0
            % no foreground marker in this region, remove it
            testL(testL==ii) = 0;
        else
            maxnum = max([maxnum,numnuc]);
        end
    end
end


if maxnum>1
    tbw = testL>0;
    tbw(tbw==0) = Inf;
    L = watershed(imimposemin(-tbw,fg3d)).*(testL>0);
    % this is very slow and should be avoided if at all possible!
else
    L = bwlabeln(testL>0);
end

nucstats = regionprops(L,'Area','PixelIdxList','Centroid','BoundingBox');

Ler = imerode(L,cat(3,[0,0,0;0,1,0;0,0,0],[0,1,0;1,1,1;0,1,0],[0,0,0;0,1,0;0,0,0]));
nucstats2 = regionprops(Ler,'PixelList');

xyz = field2val2(nucstats,'Centroid');
xyz = xyz(:,[2,1,3]);

remove = false(numel(nucstats),1);

for ii = 1:numel(nucstats)
    if ii>numel(nucstats2) || isempty(nucstats2(ii).PixelList)
        % could put a condition on minimum area here if we wanted
        remove(ii) = true;
        L(L==ii) = 0;
        continue
    end
    nucstats(ii).sumint = sum(im(nucstats(ii).PixelIdxList));
%     nucstats(ii).edgepix = findn(Ledge);
    nucstats(ii).edgepix = nucstats2(ii).PixelList;
    nucstats(ii).xyz = nucstats(ii).Centroid([2,1,3]);
    
    if texprops
        nucstats(ii).fluct = adstd(im(nucstats(ii).PixelIdxList))/admean(im(nucstats(ii).PixelIdxList));
        % could also measure Haralick matrix here for purposes of
        % identifying mitotic cells?
    end
end

nucstats(remove) = [];
xyz(remove,:) = [];
L = rescalelabels(L);
