function [xyz,L,nucstats] = segnuc5(im,sigma,decon,stdfact,texprops)

% combine the parts that work from segnuc4 and segnucagar.

if nargin<5 || isempty(texprops)
    texprops = 0;
end
if nargin<4 || isempty(stdfact)
    stdfact = 0.6;
end

if nargin<3 || isempty(decon)
    decon = 0;
end

if nargin<2 || isempty(sigma)
    sigma = 5;
end

[xdim,ydim,zdim] = size(im);

im2d = mean(im,3); % max is an alternative here, or the 3/4 order statistic?

[x,y] = makegrid2(zeros(4*sigma + 1));
template = exp(-((x-(2*sigma+1)).^2 + (y-(2*sigma+1)).^2)/sigma^2);

C = normxcorr2(template,padarray(im2d,[(2*sigma+1) (2*sigma+1)],'symmetric'));
% might be necessary to pad the array a little more (will then have to correct
% the cropping below too)

C2 = C((2+4*sigma):(xdim+1+4*sigma),(2+4*sigma):(ydim+1+4*sigma));

bg = watershed(-C2)==0; % see if this works.

C2(C2<0.6) = 0;

fguse = imregionalmax(C2);

xyinds = findn(fguse);
xyz = zeros(size(xyinds,1),3);
fg3d = false(size(im));

for ii = 1:size(xyinds,1)
    smoothint = adsmooth(squeeze(im(xyinds(ii,1),xyinds(ii,2),:)),5);
    xyz(ii,:) = [xyinds(ii,:), find(smoothint==max(smoothint),1,'first')];
    fg3d(xyz(ii,1),xyz(ii,2),xyz(ii,3)) = true;
    imreg = im(max(1,xyz(ii,1)-2):min(xdim,xyz(ii,1)+2),...
        max(1,xyz(ii,2)-2):min(ydim,xyz(ii,2)+2),...
        max(1,xyz(ii,3)-2):min(zdim,xyz(ii,3)+2));
    nucv(ii) = mean(imreg(:));
    nucstd(ii) = std(imreg(:));
end

imoc = iobrcbr(log(im2d),discel(sigma/2,1));
gradim = sobel2(imoc);

testL = watershed(imimposemin(gradim,fguse|bg));
bgvals = unique(testL(bg));

L = zeros(size(im));
Lcount = 0;
for ii = 1:size(xyinds,1)
    nuclabel = testL(xyinds(ii,1),xyinds(ii,2));
    if sum(bgvals==nuclabel)>0
        % part of the background, ignore this nucleus
        continue
    end
    thisN = testL==nuclabel;
    % need a way of checking if this nucleus lands on part of the
    % background, and if it does ignore it..
    slim = reshape(im(repmat(thisN,[1 1 size(im,3)])),[nnz(thisN),size(im,3)]);
    
%     smslim = iobrcbr(slim,true(1,5));
    % could order statistic filter this instead of morphop
    smslim = adordfilt(slim,true(3,5),0.5);
    % WHICH IS BETTER?
    
    Lcount = Lcount + 1;
    slimbw = Lcount*(smslim>(nucv(ii)-0.8*nucstd(ii)));

    L(repmat(thisN,[1,1,size(im,3)])) = slimbw(:);
end

try
L = adordfilt(L,ones(5,5));
catch ME
    rethrow(ME)
end
% morphological closing might be better than order statistic filtering, but
% probably slower..

nucstats = regionprops(L,'Area','PixelIdxList','Centroid','BoundingBox');

Ler = imerode(L,cat(3,[0,0,0;0,1,0;0,0,0],[0,1,0;1,1,1;0,1,0],[0,0,0;0,1,0;0,0,0]));
nucstats2 = regionprops(Ler,'PixelList');
xyz = field2val2(nucstats,'Centroid');
xyz(:,[2,1,3]) = xyz;

remove = false(numel(nucstats),1);

for ii = 1:numel(nucstats)
    if ii>numel(nucstats2) || isempty(nucstats2(ii).PixelList)
        % could put a condition on minimum area here if we wanted
        remove(ii) = true;
        L(L==ii) = 0;
        continue
    end
    nucstats(ii).sumint = sum(im(nucstats(ii).PixelIdxList));
    nucstats(ii).edgepix = nucstats2(ii).PixelList;
    nucstats(ii).xyz = nucstats(ii).Centroid([2,1,3]);
    
    if texprops
        nucstats(ii).fluct = adstd(im(nucstats(ii).PixelIdxList))/admean(im(nucstats(ii).PixelIdxList));
        % could also measure Haralick matrix here for purposes of
        % identifying mitotic cells? How slow would this be?
        % need to first check that it's not just the brightest parts of the
        % nucleus that get included..
    end
end

nucstats(remove) = [];
xyz(remove,:) = [];
L = rescalelabels(L);

