function [xyz,L,nucstats,th] = segnucvox(im,sigma,th,zlims)

% nuclear segmentation specifically for the case of cells under agar, where
% the nuclei tend to flatter and less circular than usual

% the vox version of this program is required to account for the sharper
% bondaries than are obtained with the wide-field microscope.  The main
% modification is to weight gradients at low intensities more highly than
% those at high intensities, to better define the edge of the nuclei.

if nargin<4 || isempty(zlims)
    zlims = [1,size(im,3)];
end
if nargin<3 || isempty(th)
    th = 'auto';
end
if nargin<2 || isempty(sigma)
    sigma = 8;
end

texprops = 1;
[xdim,ydim,zdim] = size(im);

if isscalar(zlims)
    zlims = [1,1]*zlims;
end
if zlims(1)<0
    zlims(1) = 1-zlims(1);
end
if zlims(2)<0
    zlims(2) = zdim+zlims(2);
end

im2d = mean(im(:,:,zlims(1):zlims(2)),3);

J = normalise(im2d);

fi = dog3split2(sigma,ceil(5*sigma));
fim = gauss2filt(log(im2d),fi);

% lmax = (fim-imdilate(fim,ones(ceil(1.5*sigma))))>(-0.0001);
% local maxima finder
lmax = imregionalmax(fim);

bg = watershed(-fim)==0; % see if this works.

if ischar(th)
    switch lower(th)
        case 'auto'
            % need to try to find a good threshold
            th = 0.1*max(fim(:))+ 0.9*mean(fim(bg));
        case 'manual'
            th = findthresh(fim,im2d);
    end
end

bw = fim>th;

fguse = lmax & bw; % only keep maxima above our threshold

xyinds = findn(fguse);
xyz = zeros(size(xyinds,1),3);
% % % fg3d = false(size(im));
% % % 
% % % for ii = 1:size(xyinds,1)
% % %     
% % % end

% bgv = mean(im2d(bg));
% localthresh = -Inf*ones(size(im,1),size(im,2));
% 
% for ii = 1:size(xyz,1)
%     imreg = im(max(1,xyz(ii,1)-2):min(xdim,xyz(ii,1)+2),...
%         max(1,xyz(ii,2)-2):min(ydim,xyz(ii,2)+2),...
%         max(1,xyz(ii,3)-2):min(zdim,xyz(ii,3)+2));
%     nucv = mean(imreg(:));
%     localthresh(xyz(ii,1),xyz(ii,2)) = bgv+stdfact*(nucv-bgv);
% end


% localthresh = adordfilt(im2d,ones(3*sigma),0.8);

% localthresh = repmat(imdilate(localthresh,discel(2*sigma,1)),[1,1,size(im,3)]);
% localthresh(isinf(localthresh)) = Inf;



imoc = iobrcbr(J,discel(sigma/2,1));
% level set diffusion might be a viable alternative to morphological
% reconstruction.

% gradim = (1-0.5*imoc).*adgaussgrad2(imoc,1,15);
gradim = adgaussgrad2(imoc,1,15).*exp(-imoc/0.25);

testL = watershed(imimposemin(gradim,fguse|bg));

% watershedding sets the boundary as background, rectify that here
% testL = imdilate(testL,[0,1,0;1,1,1;0,1,0]);

% % % testL = wshedi(gradim,fguse,bg);

bgvals = unique(testL(bg));
for ii = 1:numel(bgvals)
    testL(testL==bgvals(ii)) = 0;
end
testL = rescalelabels(testL);
tstats = regionprops(testL,'Centroid');

nucl = im(:,:,1);
bgl = admean(nucl(bg));

nucu = im(:,:,end);
bgu = admean(nucu(bg));

L = zeros(size(im));
for ii = 1:numel(tstats)
    try
        smoothint = adsmooth(squeeze(im(round(tstats(ii).Centroid(2)),round(tstats(ii).Centroid(1)),:)),5);
    catch ME
        rethrow(ME)
    end
    xyz(ii,:) = round([tstats(ii).Centroid([2,1]), find(smoothint==max(smoothint),1,'first')]);
%     fg3d(xyz(ii,1),xyz(ii,2),xyz(ii,3)) = true;
    imreg = im(max(1,xyz(ii,1)-2):min(xdim,xyz(ii,1)+2),...
        max(1,xyz(ii,2)-2):min(ydim,xyz(ii,2)+2),...
        max(1,xyz(ii,3)-2):min(zdim,xyz(ii,3)+2));
    nucv = mean(imreg(:));
    nucstd = std(imreg(:));
    
    nuclabel = testL(xyz(ii,1),xyz(ii,2));
    thisN = testL==nuclabel;
    slim = reshape(im(repmat(thisN,[1 1 size(im,3)])),[nnz(thisN),size(im,3)]);
    
%     smslim = iobrcbr(slim,true(1,5));
    % could order statistic filter this instead of morphop
    smslim = adordfilt(slim,true(3,5),0.5);
    % WHICH IS BETTER?
    
    slp = [bgl*ones(size(slim,1),1),smslim,bgu*ones(size(slim,1),1)];
    
    slimbw = zeros(size(smslim));
    % find the maximal gradients either side of the maxima
    zz = 1:size(smslim,2);
    for kk = 1:size(smslim,1)
        maxind = find(smslim(kk,:)==max(smslim(kk,:)),1,'first');
        sgrad = slp(kk,3:end) - slp(kk,1:(end-2));
        if maxind>1
            lval = max(sgrad(zz<maxind));
            lind = find(zz<maxind & sgrad==lval,1,'first');
        else
            lind = 1;
        end
        
        if maxind<size(smslim,2)
            uval = min(sgrad(zz>maxind));
            uind = find(zz>maxind & sgrad==uval,1,'last');
        else
            uind = size(smslim,2);
        end
        
        slimbw(kk,lind:uind) = ii;
    end
    
%     slimbw = ii*(smslim>(nucv-0.8*nucstd));

    L(repmat(thisN,[1,1,size(im,3)])) = slimbw(:);
    
    % check for fragmentation
    [temp,tN] = bwlabeln(L==ii);
    if tN>1
        % keep only the part with the largest volume
        fragstats = regionprops(temp,'Area');
        keepind = find([fragstats.Area]==max([fragstats.Area]),1,'first');
        L(L==ii)=0;
        L(temp==keepind)=ii;
%         disp('Sorted out fragmented nucleus') % to see how prevalent this is
        % might be better to write to a log instead of the screen
    end
end

L = imdilate(L,[0,1,0;1,1,1;0,1,0]);

% somewhere we need to check for fragmentation of a nuclear label into more
% than region; this causes problems with the cell segmentation.


% testL = bwlabeln(bw);

nucstats = regionprops(L,'Area','PixelIdxList','Centroid','BoundingBox');

bb = field2val2(nucstats,'BoundingBox');

% need to allow for 2D and 3D images here

switch size(bb,2)
    case 6
        roughsol = bb(:,4).*bb(:,5).*bb(:,6)./field2val2(nucstats,'Area');
    otherwise
        roughsol = bb(:,3).*bb(:,4)./field2val2(nucstats,'Area');
end


xyz = field2val2(nucstats,'Centroid');
% % xyz = xyz(:,[2,1,3]);
xyz(:,[1,2]) = xyz(:,[2,1]);

ndim = size(xyz,2);

% make sure that this line can't mess up the nuclear positions in the case
% of touching nuclei
% Ledge = L - Ler;

% do we need to make sure that the top and bottom erode inwards?
% for the purposes of finding the nuclear boundaries, perform morphological
% closing to remove noisy pixels

Ledge = imclose(L,discel(3,1));

switch ndim
    case 3
        Ler = imerode(padarray(Ledge,[0,0,1]),cat(3,[0,0,0;0,1,0;0,0,0],[0,1,0;1,1,1;0,1,0],[0,0,0;0,1,0;0,0,0]));
        Ler = Ler(:,:,2:(end-1));
    otherwise
        Ler = imerode(Ledge,[0,1,0;1,1,1;0,1,0]);
end

Ledge(Ler~=0) = 0;

nucstats2 = regionprops(Ledge,'PixelList','BoundingBox');

remove = false(numel(nucstats),1);

switch ndim
    case 3
        alim = 20*(4/3*pi*sigma^3);
        % order of magnitude limit based on our chosen value of sigma
    otherwise
        alim = 16*pi*sigma^2; % factor of 8 might be too much for two dimensions
end

[xx,yy] = makegrid2(im);

for ii = 1:numel(nucstats)
    if ii>numel(nucstats2) || isempty(nucstats2(ii).PixelList) || ...
            nucstats(ii).Area>alim || roughsol(ii)>5
        % could put a condition on minimum and maximum area here if we wanted
        remove(ii) = true;
        L(L==ii) = 0;
        continue
    end
    nucstats(ii).sumint = sum(im(nucstats(ii).PixelIdxList));
%     nucstats(ii).edgepix = findn(Ledge);
    nucstats(ii).edgepix = nucstats2(ii).PixelList;
    
    temp = nucstats(ii).Centroid;
    temp([1,2]) = temp([2,1]);
    nucstats(ii).xyz = temp; % do this the long way to allow for 2 or 3 dimensions
    
    % calculate the nuclear shape aspect ratio and orientation here so that
    % the biases of the edge don't creep in
    x2 = adsum((xx(nucstats(ii).PixelIdxList)-nucstats(ii).xyz(1)).^2)/numel(nucstats(ii).PixelIdxList);
    y2 = adsum((yy(nucstats(ii).PixelIdxList)-nucstats(ii).xyz(2)).^2)/numel(nucstats(ii).PixelIdxList);
    xy = adsum((xx(nucstats(ii).PixelIdxList)-nucstats(ii).xyz(1)).*(yy(nucstats(ii).PixelIdxList)-nucstats(ii).xyz(2)))/numel(nucstats(ii).PixelIdxList);
    
    [v,d] = eig([x2,xy;xy,y2]);
    if d(1)>d(4)
        nucstats(ii).aspr = sqrt(d(1)/d(4));
        nucstats(ii).longax = atan(v(2)/v(1));
    else
        nucstats(ii).aspr = sqrt(d(4)/d(1));
        nucstats(ii).longax = atan(v(4)/v(3));
    end
    
    
    if texprops
        nucstats(ii).fluct = adstd(im(nucstats(ii).PixelIdxList))/admean(im(nucstats(ii).PixelIdxList));
        % could also measure Haralick matrix here for purposes of
        % identifying mitotic cells?
    end
end

nucstats(remove) = [];
xyz(remove,:) = [];
L = rescalelabels(L);


