function [im,tdigs] = voltiffimportbeta(varargin)

% import tiff files which have been exported as tiffs from volocity

% TO DO - add a part of this that allows multiple time points to be read in
%       - specify as vectors which z-slices, channels, and time points to
%         read in. (the z part of this may need a rewriting of readstack)
%       - add the ability to crop in the x-y (readstack already has this)

% the inputs should be specified as pairs, the first of the pair is a
% string identifying what the input is, the second is the input.

% 'f' - path to parent folder. If omitted, use current folder
% 't' - time point (a number rather than a string)
% 'c' - channel
% 'z' - optional, z-slice. If omitted, read in the stack

% need to parse these, first load the default parameters

% the slowest step for long movies is the collection of the timepoint
% folders, but we can avoid this if we specify the number of digits in the
% folder name (assuming this is constant)
% to facilitate future calls to the function (in movie analysis for
% example), have tdigs as a second output to save having to measure it
% every time.

timeval = 1;
channel = 1;
zslice = 'all';
fpath = cd;
fileext = 'tif';

tlabel = 'T';
clabel = 'C';
zlabel = 'Z';

tdigs = [];

origpath = cd;

for ii = 1:2:length(varargin)
    if ii<length(varargin)
        % first of the pair should be a string
        switch lower(varargin{ii})
            case {'f','fpath'}
                if ischar(varargin{ii+1})
                    fpath = varargin{ii+1};
                else
                    error('Folder path must be specified as a string')
                end
            case {'t','time','frame'}
                timeval = varargin{ii+1};
            case {'c','channel'}
                channel = varargin{ii+1};
            case {'z','zslice','slice','slices','zslices'}
                zslice = varargin{ii+1};
            case {'e','ext','fileext','filetype'}
                fileext = varargin{ii+1};
            case 'tlabel'
                tlabel = varargin{ii+1};
            case 'clabel'
                clabel = varargin{ii+1};
            case 'zlabel'
                zlabel = varargin{ii+1};
            case 'tdigs'
                tdigs = varargin{ii+1};
        end
    else
        % last input, no pair, this has to be interpreted somehow
        % just crash for now
        error('Inputs must be specified as parameter-value pairs')
    end
end

% the folder path should take us to a directory which contains the
% individual time point folders

if ~strcmpi(fpath,origpath)
    cd(fpath)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DO WE USE NUMBER OF T DIGITS?

if islogical(tdigs) && tdigs
    folnames = dir(['*' tlabel '*']);
    folnames(~[folnames.isdir]) = []; % only keep folders
    if numel(folnames)==0
        error('No time folders found in specified directory')
    end
    tdigs = length(folnames(1).name) - 1; % might be not be foolproof enough
    % cd to this folder
    cd([tlabel, num2str(timeval,['%06.' num2str(tdigs) 'd'])])
    
elseif ~isempty(tdigs)
    % don't read in any folder names, just cd to what should be the name of
    % the folder.
    % should be much quicker than the other methods
    
    cd([tlabel, num2str(timeval,['%06.' num2str(tdigs) 'd'])])
    
else
    folnames = dir;
    folnames(~[folnames.isdir]) = []; % only keep folders
    if numel(folnames)==0
        error('No time folders found in specified directory')
    end

    % read the number after the 'T' for each folder, and match the one
    % requested.
    for ii = 1:numel(folnames)
        [s,f] = regexp(folnames(ii).name,[tlabel '\d{1,20}']);
        tnum(ii) = str2double(folnames(ii).name((s+length(tlabel)):f));
    %     tdigs(ii) = f-s;
    end

    % check that there's only one folder with the number we're looking for
    tind = find(tnum==timeval);
    if numel(tind)>1
        error('More than one folder has same value for time label')
    end

    % go into the folder and start again.
    cd(folnames(tind).name)
end

% shouldn't need the T-label now.

% fast method - read whether channel or z comes first, get the number of
% digits for each and use readstack.
% it's important that only the image files have the extension specified.
% filelist = dir(['*.' fileext]);

if ~isempty(zlabel)
    if ~isempty(clabel)
        filelist = [dir(['*' clabel '*' zlabel '*.' fileext]);dir(['*' zlabel '*' clabel '*.' fileext])];
        % include whether channel or z comes first
    else
        filelist = dir(['*' zlabel '*.' fileext]);
    end
else
    if ~isempty(clabel)
        filelist = dir(['*' clabel '*.' fileext]);
    else
        filelist = dir(['*.' fileext]);
    end
end

if isempty(filelist)
    error('No files found with labels specified')
end

idlabel = '*'; % could put the T##### here if we want to, but will make it less transferable
idcpart = [];
idzpart = [];
sc = Inf;
sz = Inf;

if ischar(channel)
    if strcmpi(channel,'all') || numel(channel)>1
        % read in channels as a separate dimension
        error('Not finished yet')
    end
else
    if ~isempty(clabel)
        % take the number of channel digits from the first file name
        [sc,fc] = regexp(filelist(1).name,[clabel '\d{1,10}']);
        cdigs = fc-sc+1-length(clabel);
        idcpart = [clabel num2str(channel,['%06.' num2str(cdigs) 'd']) '*'];
    end
end

if ~isempty(zlabel)
    if ~ischar(zslice) && ~isempty(zslice) && numel(zslice)==1
        [sz,fz] = regexp(filelist(1).name,[zlabel '\d{1,10}']);
        zdigs = fz-sz+1-length(zlabel);
        idzpart = [zlabel num2str(zslice,['%06.' num2str(zdigs) 'd']) '*'];
        
        %redefine zslice so it works in the new readstackbeta
        zslice = 'all';
    else
        idzpart = [zlabel '*']; % take all z-slices
    end
end


% now find whether channel or z comes first
if sc<sz
    idlabel = [idlabel idcpart idzpart]; %'.' fileext]; % fileext is done separately by readstack
else
    idlabel = [idlabel idzpart idcpart]; %'.' fileext];
end

% should now be complete to read in.  If no zslice was specified, or it was
% given as 'all', there should be no z part in the idlabel, so readstack
% should get the whole stack.

% It would be better to require the zlabel if zslice was specified as
% 'all', that way we wouldn't read in anything which isn't part of the
% stack. FIXED

% im = readstack(fileext,idlabel);
im = readstackbeta('type',fileext,'label',idlabel,'zslice',zslice);

if ~strcmpi(fpath,origpath)
    cd(origpath)
else
    cd ..
end









